@@ -33,18 +33,20 @@
 #include "config.h"
 #include <netdb.h>
 #include "capture.h"
+#include "capture_ws.h"
 #ifdef WITH_OPENSSL
 #include "capture_tls.h"
 #endif
 #include "sip.h"
-#include "option.h"
+#include "setting.h"
 #include "ui_manager.h"
 #ifdef WITH_IPV6
 #include <netinet/ip6.h>
 #endif
 
 // Capture information
-capture_info_t capinfo = { 0 };
+capture_info_t capinfo =
+    { 0 };
 
 int
 capture_online(const char *dev, const char *outfile)
@@ -157,9 +159,9 @@ parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packe
     // Fragmentation offset
     uint16_t ip_frag_off = 0;
     //! Source Address
-    char ip_src[INET6_ADDRSTRLEN + 1];
+    char ip_src[ADDRESSLEN];
     //! Destination Address
-    char ip_dst[INET6_ADDRSTRLEN + 1];
+    char ip_dst[ADDRESSLEN];
     // UDP header data
     struct udphdr *udp;
     // UDP header size
@@ -180,6 +182,8 @@ parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packe
     int transport; /* 0 UDP, 1 TCP, 2 TLS */
     // Source and Destination Ports
     u_short sport, dport;
+    // Media structure for RTP packets
+    rtp_stream_t *stream;
 
     // Ignore packets while capture is paused
     if (capture_is_paused())
@@ -197,13 +201,13 @@ parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packe
 
 #ifdef WITH_IPV6
     // Get IPv6 header
-    ip6 = (struct ip6_hdr*)(packet + size_link);
+    ip6 = (struct ip6_hdr*) (packet + size_link);
 #endif
 
     // Get IP version
     ip_ver = ip4->ip_v;
 
-    switch(ip_ver) {
+    switch (ip_ver) {
         case 4:
             size_ip = ip4->ip_hl * 4;
             ip_proto = ip4->ip_p;
@@ -243,11 +247,11 @@ parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packe
         udp_size = (ip_frag_off) ? 0 : sizeof(struct udphdr);
 
         // Set packet ports
-        sport = udp->uh_sport;
-        dport = udp->uh_dport;
+        sport = htons(udp->uh_sport);
+        dport = htons(udp->uh_dport);
 
         size_payload = htons(udp->uh_ulen) - udp_size;
-        if ((int32_t)size_payload > 0 ) {
+        if ((int32_t) size_payload > 0) {
             // Get packet payload
             msg_payload = malloc(size_payload + 1);
             memset(msg_payload, 0, size_payload + 1);
@@ -265,12 +269,12 @@ parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packe
         tcp_size = (ip_frag_off) ? 0 : (tcp->th_off * 4);
 
         // Set packet ports
-        sport = tcp->th_sport;
-        dport = tcp->th_dport;
+        sport = htons(tcp->th_sport);
+        dport = htons(tcp->th_dport);
 
         // We're only interested in packets with payload
         size_payload = ip_len - (size_ip + tcp_size);
-        if ((int32_t)size_payload > 0) {
+        if ((int32_t) size_payload > 0) {
             // Get packet payload
             msg_payload = malloc(size_payload + 1);
             memset(msg_payload, 0, size_payload + 1);
@@ -290,58 +294,57 @@ parse_packet(u_char *mode, const struct pcap_pkthdr *header, const u_char *packe
                 tls_process_segment(ip4, &msg_payload, &size_payload);
 
                 // Check if we have decoded payload
-                if (size_payload <= 0)
+                if ((int32_t) size_payload <= 0) {
                     free(msg_payload);
+                    return;
+                }
 
                 // Set Transport TLS
                 transport = 2;
             }
         }
 #endif
+        // Check if packet is Websocket
+        if (msg_payload && capture_ws_check_packet(msg_payload, &size_payload)) {
+            transport = 3;
+        }
     } else {
         // Not handled protocol
         return;
     }
 
-    // Increase capture stats
-    if (ip4->ip_v == 4 && capinfo.devices) {
-        if(is_local_address(ip4->ip_src.s_addr)) {
-            capinfo.local_ports[htons(sport)]++;
-            capinfo.remote_ports[htons(dport)]++;
-        } else {
-            capinfo.remote_ports[htons(sport)]++;
-            capinfo.local_ports[htons(dport)]++;
-        }
-    }
-
     // We're only interested in packets with payload
-    if (size_payload <= 0)
+    if ((int32_t) size_payload <= 0)
         return;
 
     // Parse this header and payload
-    msg = sip_load_message(header, ip_src, sport, ip_dst, dport, msg_payload);
-    free(msg_payload);
+    if ((msg = sip_load_message(header, ip_src, sport, ip_dst, dport, msg_payload))) {
+        // Store Transport attribute
+        if (transport == 0) {
+            msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "UDP");
+        } else if (transport == 1) {
+            msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TCP");
+        } else if (transport == 2) {
+            msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TLS");
+        } else if (transport == 3) {
+            msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "WS");
+        }
 
-    // This is not a sip message, Bye!
-    if (!msg)
-        return;
+        // Set message PCAP data
+        msg->pcap_packet = malloc(size_packet);
+        memcpy(msg->pcap_packet, packet, size_packet);
 
-    // Store Transport attribute
-    if (transport == 0) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "UDP");
-    } else if (transport == 1) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TCP");
-    } else if (transport == 2) {
-        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TLS");
+        // Store this packets in output file
+        dump_packet(capinfo.pd, header, packet);
+    } else {
+        if ((stream = rtp_check_stream(header, ip_src, sport, ip_dst, dport, msg_payload))) {
+            // Store this packets in output file
+            dump_packet(capinfo.pd, header, packet);
+        }
     }
 
-    // Set message PCAP data
-    msg->pcap_packet = malloc(size_packet);
-    memcpy(msg->pcap_packet, packet, size_packet);
-
-    // Store this packets in output file
-    dump_packet(capinfo.pd, header, packet);
-
+    // Deallocate packet duplicated payload
+    free(msg_payload);
 }
 
 void
@@ -436,7 +439,7 @@ capture_get_status()
 const char *
 capture_get_status_desc()
 {
-    switch(capinfo.status) {
+    switch (capinfo.status) {
         case CAPTURE_ONLINE:
             return "Online";
         case CAPTURE_ONLINE_PAUSED:
@@ -477,7 +480,7 @@ int
 datalink_size(int datalink)
 {
     // Datalink header size
-    switch(datalink) {
+    switch (datalink) {
         case DLT_EN10MB:
             return 14;
         case DLT_IEEE802:
@@ -542,9 +545,14 @@ lookup_hostname(const char *address)
 {
     int i;
     int hostlen;
+    in_addr_t netaddress;
     struct hostent *host;
     const char *hostname;
 
+    // No lookup enabled, return address as is
+    if (!setting_enabled(SETTING_CAPTURE_LOOKUP))
+        return address;
+
     // Check if we have already tryied resolve this address
     for (i = 0; i < capinfo.dnscache.count; i++) {
         if (!strcmp(capinfo.dnscache.addr[i], address)) {
@@ -552,8 +560,12 @@ lookup_hostname(const char *address)
         }
     }
 
+    // Convert the address to network byte order
+    if ((netaddress = inet_addr(address)) == -1)
+        return address;
+
     // Lookup this addres
-    host = gethostbyaddr(address, 4, AF_INET);
+    host = gethostbyaddr(&netaddress, sizeof(netaddress), AF_INET);
     if (!host) {
         hostname = address;
     } else {
@@ -586,17 +598,9 @@ is_local_address(in_addr_t address)
 
     for (device = capinfo.devices; device; device = device->next) {
         for (dev_addr = device->addresses; dev_addr; dev_addr = dev_addr->next)
-            if (dev_addr->addr && dev_addr->addr->sa_family == AF_INET && ((struct sockaddr_in*)dev_addr->addr)->sin_addr.s_addr == address)
+            if (dev_addr->addr && dev_addr->addr->sa_family == AF_INET
+                    && ((struct sockaddr_in*) dev_addr->addr)->sin_addr.s_addr == address)
                 return 1;
     }
     return 0;
 }
-
-int
-capture_packet_count_port(int type, int port)
-{
-    if (type == 0)
-        return capinfo.remote_ports[port];
-    else
-        return capinfo.local_ports[port];
-}
