@@ -39,7 +39,6 @@
 #include <stdlib.h>
 #include <time.h>
 
-#if defined(__linux__)
 #ifndef __FAVOR_BSD
 #define __FAVOR_BSD
 #endif
@@ -47,16 +46,14 @@
 #ifndef _BSD_SOURCE
 #define _BSD_SOURCE 1
 #endif
-#endif
 
-#if defined(BSD) || defined (__OpenBSD__)
-#define bpf_timeval timeval
+#if defined(BSD) || defined (__OpenBSD__) || defined(__FreeBSD__)
+#define timeval bpf_timeval
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <net/if.h>
-#include <arpa/inet.h>
 #endif
 
 #include <arpa/inet.h>
@@ -65,6 +62,13 @@
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
 
+#ifdef INET6_ADDRSTRLEN
+#define ADDRESSLEN INET6_ADDRSTRLEN + 1
+#else
+#define ADDRESSLEN 47
+#endif
+
+
 //! Capture modes
 enum capture_status {
     CAPTURE_ONLINE = 0,
@@ -86,7 +90,7 @@ typedef struct dns_cache dns_cache_t;
  */
 struct dns_cache {
     int count;
-    char addr[16][256];
+    char addr[ADDRESSLEN][256];
     char hostname[16][256];
 };
 
@@ -121,10 +125,6 @@ struct capture_info {
     dns_cache_t dnscache;
     //! Local devices pointer
     pcap_if_t *devices;
-    //! Incoming Packets per port
-    int local_ports[65535];
-    //! Outgoint Packets per port
-    int remote_ports[65535];
     //! Capture thread for online capturing
     pthread_t capture_t;
 };
@@ -328,13 +328,4 @@ is_local_address_str(const char *address);
 int
 is_local_address(in_addr_t address);
 
-/**
- * @brief Return the packet count on a given port
- *
- * @param type 1 for local ports, 0 for remote
- * @return the packet count on requested port
- */
-int
-capture_packet_count_port(int type, int port);
-
 #endif
