@@ -28,7 +28,6 @@
  * @todo Replace structures for their typedef shorter names
  */
 #include "config.h"
-#include <regex.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -38,7 +37,7 @@
 #include <stdarg.h>
 #include "sip.h"
 #include "option.h"
-#include "capture.h"
+#include "setting.h"
 #include "filter.h"
 
 /**
@@ -47,7 +46,8 @@
  * All parsed calls will be added to this list, only accesible from
  * this awesome structure, so, keep it thread-safe.
  */
-static sip_call_list_t calls = { 0 };
+sip_call_list_t calls =
+    { 0 };
 
 void
 sip_init(int limit, int only_calls, int no_incomplete)
@@ -59,6 +59,9 @@ sip_init(int limit, int only_calls, int no_incomplete)
     calls.only_calls = only_calls;
     calls.ignore_incomplete = no_incomplete;
 
+    // Create a vector to store calls
+    calls.list = vector_create(500, 50);
+
     // Create hash table for callid search
     hcreate(calls.limit);
 
@@ -74,15 +77,16 @@ sip_init(int limit, int only_calls, int no_incomplete)
 
     // Initialize payload parsing regexp
     match_flags = REG_EXTENDED | REG_ICASE | REG_NEWLINE;
-    regcomp(&calls.reg_method, "^([a-zA-Z]+) sip:\\S+ SIP/2.0\r", match_flags & ~REG_NEWLINE);
-    regcomp(&calls.reg_callid, "^(Call-ID|i):\\s*(\\S+)\r$", match_flags);
-    regcomp(&calls.reg_xcallid, "^(X-Call-ID|X-CID):\\s*(\\S+)\r$", match_flags);
-    regcomp(&calls.reg_response, "^SIP/2.0\\s*(([0-9]{3}) [^\r]+)\r", match_flags & ~REG_NEWLINE);
-    regcomp(&calls.reg_cseq, "^CSeq:\\s*([0-9]+) .+\r$", match_flags);
-    regcomp(&calls.reg_from, "^(From|f):\\s*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
-    regcomp(&calls.reg_to, "^(To|t):\\s*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
-    regcomp(&calls.reg_sdp_addr, "^c=\\S+ \\S+ (.+)\r$", match_flags);
-    regcomp(&calls.reg_sdp_port, "^m=\\S+ ([0-9]+)", match_flags);
+    regcomp(&calls.reg_method, "^([a-zA-Z]+) sip:[^ ]+ SIP/2.0\r", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_callid, "^(Call-ID|i):[ ]*([^ ]+)\r$", match_flags);
+    regcomp(&calls.reg_xcallid, "^(X-Call-ID|X-CID):[ ]*([^ ]+)\r$", match_flags);
+    regcomp(&calls.reg_response, "^SIP/2.0[ ]*(([0-9]{3}) [^\r]+)\r", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_cseq, "^CSeq:[ ]*([0-9]+) .+\r$", match_flags);
+    regcomp(&calls.reg_from, "^(From|f):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
+    regcomp(&calls.reg_to, "^(To|t):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
+    regcomp(&calls.reg_sdp, "^Content-Type:[ ]* application/sdp\r$", match_flags);
+    regcomp(&calls.reg_sdp_addr, "^c=[^ ]+ [^ ]+ (.+)\r$", match_flags);
+    regcomp(&calls.reg_sdp_port, "^m=[^ ]+ ([0-9]+)", match_flags);
 
 }
 
@@ -96,27 +100,18 @@ sip_msg_create(const char *payload)
     memset(msg, 0, sizeof(sip_msg_t));
     msg->payload = strdup(payload);
     msg->color = 0;
+    // Create a vector to store sdp
+    msg->medias = vector_create(0, 2);
     return msg;
 }
 
 void
 sip_msg_destroy(sip_msg_t *msg)
 {
-    sip_msg_t *prev = NULL;
 
     if (!msg)
         return;
 
-    // If the message belongs to a call, remove it from
-    // its message list
-    if (msg->call) {
-        if ((prev = call_get_prev_msg(msg->call, msg))) {
-            prev->next = msg->next;
-        } else {
-            msg->call->msgs = msg->next;
-        }
-    }
-
     // Free message attribute list
     sip_attr_list_destroy(msg->attrs);
 
@@ -138,29 +133,32 @@ sip_call_t *
 sip_call_create(char *callid)
 {
     ENTRY entry;
+    int index;
+
     // Initialize a new call structure
     sip_call_t *call = malloc(sizeof(sip_call_t));
     memset(call, 0, sizeof(sip_call_t));
 
-    if (!calls.count) {
-        calls.first = call;
-    } else {
-        call->prev = calls.last;
-        calls.last->next = call;
-    }
-    calls.last = call;
-    calls.count++;
+    // Add this call to the call list
+    pthread_mutex_lock(&calls.lock);
+    index = vector_append(calls.list, call);
+    pthread_mutex_unlock(&calls.lock);
 
     // Store this call in hash table
     entry.key = strdup(callid);
     entry.data = (void *) call;
     hsearch(entry, ENTER);
 
+    // Create a vector to store call messages
+    call->msgs = vector_create(10, 5);
+    // Create a vector to store RTP streams
+    call->streams = vector_create(4, 2);
+
     // Initialize call filter status
     call->filtered = -1;
 
     // Store current call Index
-    call_set_attribute(call, SIP_ATTR_CALLINDEX, "%d", calls.count);
+    call_set_attribute(call, SIP_ATTR_CALLINDEX, "%d", index);
 
     return call;
 }
@@ -168,28 +166,23 @@ sip_call_create(char *callid)
 void
 sip_call_destroy(sip_call_t *call)
 {
+    sip_msg_t *msg;
+    vector_iter_t msgs;
+
     // No call to destroy
     if (!call)
         return;
 
-    // If removing the first call, update list head
-    if (call == calls.first)
-        calls.first = call->next;
-    // If removing the last call, update the list tail
-    if (call == calls.last)
-        calls.last = call->prev;
-    // Update previous call
-    if (call->prev)
-        call->prev->next = call->next;
-    // Update next call
-    if (call->next)
-        call->next->prev = call->prev;
-    // Update call counter
-    calls.count--;
+    // Remove call from calls list
+    vector_remove(calls.list, call);
+
+    // Remove all call messages
+    msgs = vector_iterator(call->msgs);
+    while ((msg = vector_iterator_next(&msgs))) {
+        sip_msg_destroy(msg);
+        vector_remove(call->msgs, msg);
+    }
 
-    // Remove all messages
-    while (call->msgs)
-        sip_msg_destroy(call->msgs);
 
     // Remove all call attributes
     sip_attr_list_destroy(call->attrs);
@@ -218,8 +211,8 @@ sip_get_callid(const char* payload)
 }
 
 sip_msg_t *
-sip_load_message(const struct pcap_pkthdr *header, const char *src, u_short sport,
-                 const char* dst, u_short dport, u_char *payload)
+sip_load_message(const struct pcap_pkthdr *header, const char *src, u_short sport, const char* dst,
+                 u_short dport, u_char *payload)
 {
     sip_msg_t *msg;
     sip_call_t *call;
@@ -324,6 +317,10 @@ sip_load_message(const struct pcap_pkthdr *header, const char *src, u_short spor
     // Add the message to the found/created call
     call_add_message(call, msg);
 
+    msg_parse_payload(msg, msg->payload);
+    // Parse media data
+    msg_parse_media(msg);
+
     // Update Call State
     call_update_state(call, msg);
 
@@ -336,7 +333,27 @@ sip_load_message(const struct pcap_pkthdr *header, const char *src, u_short spor
 int
 sip_calls_count()
 {
-    return calls.count;
+    return vector_count(calls.list);
+}
+
+vector_iter_t
+sip_calls_iterator()
+{
+    return vector_iterator(calls.list);
+}
+
+void
+sip_calls_stats(int *total, int *displayed)
+{
+    vector_iter_t it = vector_iterator(calls.list);
+
+    pthread_mutex_lock(&calls.lock);
+    // Total number of calls without filtering
+    *total = vector_iterator_count(&it);
+    // Total number of calls after filtering
+    vector_iterator_set_filter(&it, filter_check_call);
+    *displayed = vector_iterator_count(&it);
+    pthread_mutex_unlock(&calls.lock);
 }
 
 void
@@ -344,20 +361,10 @@ call_add_message(sip_call_t *call, sip_msg_t *msg)
 {
     // Set the message owner
     msg->call = call;
-
     // Put this msg at the end of the msg list
-    if (!call->msgs) {
-        call->msgs = msg;
-    } else {
-        call->last_msg->next = msg;
-    }
-    call->last_msg = msg;
-
-    // Increase message count
-    call->msgcnt++;
-
+    vector_append(call->msgs, msg);
     // Store message count
-    call_set_attribute(call, SIP_ATTR_MSGCNT, "%d", call->msgcnt);
+    call_set_attribute(call, SIP_ATTR_MSGCNT, "%d", vector_count(call->msgs));
 }
 
 sip_call_t *
@@ -376,14 +383,15 @@ sip_call_t *
 call_find_by_xcallid(const char *xcallid)
 {
     const char *cur_xcallid;
+    sip_call_t *cur;
+    int i;
 
-    sip_call_t *cur = calls.first;
-    while (cur) {
+    for (i=0; i < vector_count(calls.list); i++) {
+        cur = vector_item(calls.list, i);
         cur_xcallid = call_get_attribute(cur, SIP_ATTR_XCALLID);
         if (cur_xcallid && !strcmp(cur_xcallid, xcallid)) {
             return cur;
         }
-        cur = cur->next;
     }
     return NULL;
 }
@@ -391,7 +399,13 @@ call_find_by_xcallid(const char *xcallid)
 int
 call_msg_count(sip_call_t *call)
 {
-    return call->msgcnt;
+    return vector_count(call->msgs);
+}
+
+int
+msg_media_count(sip_msg_t *msg)
+{
+    return vector_count(msg->medias);
 }
 
 sip_call_t *
@@ -414,9 +428,9 @@ call_get_next_msg(sip_call_t *call, sip_msg_t *msg)
     sip_msg_t *ret;
     pthread_mutex_lock(&calls.lock);
     if (msg == NULL) {
-        ret = call->msgs;
+        ret = vector_first(call->msgs);
     } else {
-        ret = msg->next;
+        ret = vector_item(call->msgs, vector_index(call->msgs, msg) + 1);
     }
 
     // Parse message if not parsed
@@ -430,19 +444,9 @@ call_get_next_msg(sip_call_t *call, sip_msg_t *msg)
 sip_msg_t *
 call_get_prev_msg(sip_call_t *call, sip_msg_t *msg)
 {
-    sip_msg_t *ret = NULL, *cur;
+    sip_msg_t *ret = NULL;
     pthread_mutex_lock(&calls.lock);
-    if (msg == NULL) {
-        // No message, no previous
-        ret = NULL;
-    } else {
-        // Get previous message
-        for (cur = call->msgs; cur; ret = cur, cur = cur->next) {
-            // If cur is the message, ret will be the previous one
-            if (cur == msg)
-                break;
-        }
-    }
+    ret = vector_item(call->msgs, vector_index(call->msgs, msg) - 1);
 
     // Parse message if not parsed
     if (ret && !ret->parsed)
@@ -452,62 +456,22 @@ call_get_prev_msg(sip_call_t *call, sip_msg_t *msg)
     return ret;
 }
 
-sip_call_t *
-call_get_next(sip_call_t *cur)
+int
+call_is_active(void *item)
 {
-
-    sip_call_t * next;
-    pthread_mutex_lock(&calls.lock);
-    if (!cur) {
-        next = calls.first;
-    } else {
-        next = cur->next;
-    }
-    pthread_mutex_unlock(&calls.lock);
-    return next;
+    // TODO
+    sip_call_t *call = (sip_call_t *)item;
+    return call->active;
 }
 
-sip_call_t *
-call_get_prev(sip_call_t *cur)
+int
+msg_has_sdp(void *item)
 {
-
-    sip_call_t *prev;
-    pthread_mutex_lock(&calls.lock);
-    if (!cur) {
-        prev = calls.first;
-    } else {
-        prev = cur->prev;
-    }
-    pthread_mutex_unlock(&calls.lock);
-    return prev;
+    // TODO
+    sip_msg_t *msg = (sip_msg_t *)item;
+    return msg->sdp;
 }
 
-sip_call_t *
-call_get_next_filtered(sip_call_t *cur)
-{
-    sip_call_t *next = call_get_next(cur);
-
-    pthread_mutex_lock(&calls.lock);
-    // Return next not filtered call
-    if (next && filter_check_call(next))
-        next = call_get_next_filtered(next);
-    pthread_mutex_unlock(&calls.lock);
-
-    return next;
-}
-
-sip_call_t *
-call_get_prev_filtered(sip_call_t *cur)
-{
-    sip_call_t *prev = call_get_prev(cur);
-
-    pthread_mutex_lock(&calls.lock);
-    // Return previous call if this one is filtered
-    if (prev && filter_check_call(prev))
-        prev = call_get_prev_filtered(prev);
-    pthread_mutex_unlock(&calls.lock);
-    return prev;
-}
 
 void
 call_update_state(sip_call_t *call, sip_msg_t *msg)
@@ -515,13 +479,17 @@ call_update_state(sip_call_t *call, sip_msg_t *msg)
     const char *callstate;
     char dur[20];
     int reqresp;
+    sip_msg_t *first;
 
     // Sanity check
     if (!call || !call->msgs || !msg)
         return;
 
+    // Get the first message in the call
+    first = vector_first(call->msgs);
+
     // Check First message of Call has INVITE method
-    if (call->msgs->reqresp != SIP_METHOD_INVITE) {
+    if (first->reqresp != SIP_METHOD_INVITE) {
         return;
     }
 
@@ -535,38 +503,45 @@ call_update_state(sip_call_t *call, sip_msg_t *msg)
                 // Alice and Bob are talking
                 call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_INCALL);
                 // Store the timestap where call has started
+                call->active = 1;
                 call->cstart_msg = msg;
             } else if (reqresp == SIP_METHOD_CANCEL) {
                 // Alice is not in the mood
                 call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_CANCELLED);
                 // Store total call duration
-                call_set_attribute(call, SIP_ATTR_TOTALDUR, sip_calculate_duration(call->msgs, msg, dur));
+                call_set_attribute(call, SIP_ATTR_TOTALDUR, timeval_to_duration(first->pcap_header->ts, msg->pcap_header->ts, dur));
+                call->active = 0;
             } else if (reqresp > 400) {
                 // Bob is not in the mood
                 call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_REJECTED);
                 // Store total call duration
-                call_set_attribute(call, SIP_ATTR_TOTALDUR, sip_calculate_duration(call->msgs, msg, dur));
+                call_set_attribute(call, SIP_ATTR_TOTALDUR, timeval_to_duration(first->pcap_header->ts, msg->pcap_header->ts, dur));
+                call->active = 0;
             }
         } else if (!strcmp(callstate, SIP_CALLSTATE_INCALL)) {
             if (reqresp == SIP_METHOD_BYE) {
                 // Thanks for all the fish!
                 call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_COMPLETED);
                 // Store Conversation duration
-                call_set_attribute(call, SIP_ATTR_CONVDUR, sip_calculate_duration(call->cstart_msg, msg, dur));
+                call_set_attribute(call, SIP_ATTR_CONVDUR,
+                                   timeval_to_duration(call->cstart_msg->pcap_header->ts, msg->pcap_header->ts, dur));
+                call->active = 0;
             }
         } else if (reqresp == SIP_METHOD_INVITE && strcmp(callstate, SIP_CALLSTATE_INCALL)) {
             // Call is being setup (after proper authentication)
             call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_CALLSETUP);
+            call->active = 1;
         } else {
             // Store total call duration
-            call_set_attribute(call, SIP_ATTR_TOTALDUR, sip_calculate_duration(call->msgs, msg, dur));
+            call_set_attribute(call, SIP_ATTR_TOTALDUR, timeval_to_duration(first->pcap_header->ts, msg->pcap_header->ts, dur));
         }
     } else {
         // This is actually a call
-        if (reqresp == SIP_METHOD_INVITE)
+        if (reqresp == SIP_METHOD_INVITE) {
             call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_CALLSETUP);
+            call->active = 1;
+        }
     }
-
 }
 
 int
@@ -589,7 +564,8 @@ msg_get_reqresp(sip_msg_t *msg)
 
         // Response code
         if (regexec(&calls.reg_response, payload, 3, pmatch, 0) == 0) {
-            msg_set_attribute(msg, SIP_ATTR_METHOD, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
+            msg_set_attribute(msg, SIP_ATTR_METHOD, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so,
+                              payload + pmatch[1].rm_so);
             sprintf(reqresp, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so, payload + pmatch[2].rm_so);
         }
 
@@ -621,55 +597,33 @@ msg_parse_payload(sip_msg_t *msg, const char *payload)
 
     // X-Call-Id
     if (regexec(&calls.reg_xcallid, payload, 3, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_XCALLID, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so, payload + pmatch[2].rm_so);
+        msg_set_attribute(msg, SIP_ATTR_XCALLID, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
+                          payload + pmatch[2].rm_so);
     }
 
     // From
     if (regexec(&calls.reg_from, payload, 4, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SIPFROM, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so, payload + pmatch[2].rm_so);
-        msg_set_attribute(msg, SIP_ATTR_SIPFROMUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so, payload + pmatch[3].rm_so);
+        msg_set_attribute(msg, SIP_ATTR_SIPFROM, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
+                          payload + pmatch[2].rm_so);
+        msg_set_attribute(msg, SIP_ATTR_SIPFROMUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so,
+                          payload + pmatch[3].rm_so);
     }
 
     // To
     if (regexec(&calls.reg_to, payload, 4, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SIPTO, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so, payload + pmatch[2].rm_so);
-        msg_set_attribute(msg, SIP_ATTR_SIPTOUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so, payload + pmatch[3].rm_so);
-    }
-
-    // SDP Address
-    if (regexec(&calls.reg_sdp_addr, payload, 2, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SDP_ADDRESS, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
-        msg->sdp = 1;
-    }
-
-    // SDP Port
-    if (regexec(&calls.reg_sdp_port, payload, 2, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SDP_PORT, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
-        msg->sdp = 1;
+        msg_set_attribute(msg, SIP_ATTR_SIPTO, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
+                          payload + pmatch[2].rm_so);
+        msg_set_attribute(msg, SIP_ATTR_SIPTOUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so,
+                          payload + pmatch[3].rm_so);
     }
 
     // Set Source and Destination attributes
-    msg_set_attribute(msg, SIP_ATTR_SRC, "%s:%u", msg->src, htons(msg->sport));
-    msg_set_attribute(msg, SIP_ATTR_DST, "%s:%u", msg->dst, htons(msg->dport));
+    msg_set_attribute(msg, SIP_ATTR_SRC, "%s:%u", msg->src, msg->sport);
+    msg_set_attribute(msg, SIP_ATTR_DST, "%s:%u", msg->dst, msg->dport);
 
-    // Set Source and Destination lookpued hosts
-    if (is_option_enabled("capture.lookup")) {
-        msg_set_attribute(msg, SIP_ATTR_SRC_HOST, "%.15s:%u", lookup_hostname(msg->src), htons(msg->sport));
-        msg_set_attribute(msg, SIP_ATTR_DST_HOST, "%.15s:%u", lookup_hostname(msg->dst), htons(msg->dport));
-    }
-    msg_set_attribute(msg, SIP_ATTR_SRC_HOST, "%s:%u", msg->src, htons(msg->sport));
-    msg_set_attribute(msg, SIP_ATTR_DST_HOST, "%s:%u", msg->dst, htons(msg->dport));
-
-    // Set message Date attribute
-    time_t t = (time_t) msg->pcap_header->ts.tv_sec;
-    struct tm *timestamp = localtime(&t);
-    strftime(date, sizeof(date), "%Y/%m/%d", timestamp);
-    msg_set_attribute(msg, SIP_ATTR_DATE, date);
-
-    // Set message Time attribute
-    strftime(time, sizeof(time), "%H:%M:%S", timestamp);
-    sprintf(time + 8, ".%06d", (int) msg->pcap_header->ts.tv_usec);
-    msg_set_attribute(msg, SIP_ATTR_TIME, time);
+    // Set message Date and Time attribute
+    msg_set_attribute(msg, SIP_ATTR_DATE, timeval_to_date(msg->pcap_header->ts, date));
+    msg_set_attribute(msg, SIP_ATTR_TIME, timeval_to_time(msg->pcap_header->ts, time));
 
     // Message payload has been parsed
     msg->parsed = 1;
@@ -677,27 +631,109 @@ msg_parse_payload(sip_msg_t *msg, const char *payload)
     return 0;
 }
 
+void
+msg_parse_media(sip_msg_t *msg)
+{
+    regmatch_t pmatch[4];
+    char address[ADDRESSLEN];
+    char media_address[ADDRESSLEN] = { };
+    char media_type[15] = { };
+    char media_format[30] = { };
+    int media_port;
+    int media_fmt_pref;
+    int media_fmt_code;
+    sdp_media_t *media = NULL;
+    int port = 0;
+    char *payload, *tofree, *line;
+
+    // Check if this message has sdp
+    if (regexec(&calls.reg_sdp, msg->payload, 0, 0, 0) != 0)
+        return;
+
+    // Initialize variables
+    memset(address, 0, sizeof(address));
+
+    // SDP Address
+    if (regexec(&calls.reg_sdp_addr, msg->payload, 2, pmatch, 0) == 0) {
+        sprintf(address, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, msg->payload + pmatch[1].rm_so);
+        msg_set_attribute(msg, SIP_ATTR_SDP_ADDRESS, address);
+    }
+
+    // SDP Port
+    if (regexec(&calls.reg_sdp_port, msg->payload, 2, pmatch, 0) == 0) {
+        msg_set_attribute(msg, SIP_ATTR_SDP_PORT, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so,
+                          msg->payload + pmatch[1].rm_so);
+        port = atoi(msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
+    }
+
+    if (!strlen(address) || !port)
+        return;
+
+    // Message has SDP
+    msg->sdp = 1;
+
+    // Parse each line of payload looking for sdp information
+    tofree = payload = strdup(msg->payload);
+    while ((line = strsep(&payload, "\r\n")) != NULL) {
+        // Check if we have a media string
+        if (!strncmp(line, "m=", 2)) {
+            if (sscanf(line, "m=%s %d RTP/AVP %d", media_type, &media_port, &media_fmt_pref) == 3) {
+                // Create a new media structure for this message
+                if ((media = media_create(msg))) {
+                    media_set_type(media, media_type);
+                    media_set_port(media, media_port);
+                    media_set_address(media, media_address);
+                    media_set_format_code(media, media_fmt_pref);
+                    vector_append(msg->medias, media);
+
+                    /**
+                     * From SDP we can only guess destination address port. RTP Capture process
+                     * will determine when the stream has been completed, getting source address
+                     * and port of the stream.
+                     */
+                    // Create a new stream with this destination address:port
+                    if (!msg_is_retrans(msg)) {
+                        vector_append(msg->call->streams, stream_create(media, media_address, media_port));
+                    }
+                }
+            }
+        }
+
+        // Check if we have a connection string
+        if (!strncmp(line, "c=", 2)) {
+            if (sscanf(line, "c=IN IP4 %s", media_address) && media) {
+                    media_set_address(media, media_address);
+            }
+        }
+
+        // Check if we have attribute format string
+        if (!strncmp(line, "a=rtpmap:", 9)) {
+            if (sscanf(line, "a=rtpmap:%d %[^ ]", &media_fmt_code, media_format)) {
+                if (media && media_fmt_pref == media_fmt_code) {
+                    media_set_format(media, media_format);
+                }
+            }
+        }
+    }
+    free(tofree);
+}
+
 int
 msg_is_retrans(sip_msg_t *msg)
 {
     sip_msg_t *prev = NULL;
+    vector_iter_t it;
 
     // Sanity check
     if (!msg || !msg->call || !msg->payload)
         return 0;
 
-    // Start on previous message
-    prev = msg;
+    // Get previous message in call
+    it = vector_iterator(msg->call->msgs);
+    vector_iterator_set_current(&it, vector_index(msg->call->msgs, msg));
+    prev = vector_iterator_prev(&it);
 
-    // Check previous messages in same call
-    while ((prev = call_get_prev_msg(msg->call, prev))) {
-        // Check if the payload is exactly the same
-        if (!strcasecmp(msg->payload, prev->payload)) {
-            return 1;
-        }
-    }
-
-    return 0;
+    return (prev && !strcasecmp(msg->payload, prev->payload));
 }
 
 int
@@ -714,27 +750,31 @@ msg_get_header(sip_msg_t *msg, char *out)
 
     // We dont use Message attributes here because it contains truncated data
     // This should not overload too much as all results should be already cached
-    if (is_option_enabled("capture.lookup") && is_option_enabled("sngrep.displayhost")) {
-        sprintf(from_addr, "%s:%u", lookup_hostname(msg->src), htons(msg->sport));
-        sprintf(to_addr, "%s:%u", lookup_hostname(msg->dst), htons(msg->dport));
-    } else {
-        sprintf(from_addr, "%s:%u", msg->src, htons(msg->sport));
-        sprintf(to_addr, "%s:%u", msg->dst, htons(msg->dport));
-    }
+    sprintf(from_addr, "%s:%u", sip_address_format(msg->src), msg->sport);
+    sprintf(to_addr, "%s:%u", sip_address_format(msg->dst), msg->dport);
 
     // Get msg header
     sprintf(out, "%s %s %s -> %s", DATE(msg), TIME(msg), from_addr, to_addr);
     return out;
 }
 
+struct timeval
+msg_get_time(sip_msg_t *msg)
+{
+    if (msg) {
+        return msg->pcap_header->ts;
+    } else {
+        struct timeval t = { };
+        return t;
+    }
+}
+
 void
 sip_calls_clear()
 {
-    // Remove first call until no first call exists
     pthread_mutex_lock(&calls.lock);
-    while (calls.first) {
-        sip_call_destroy(calls.first);
-    }
+    // Clear all elements in vector
+    vector_clear(calls.list);
     // Create again the callid hash table
     hdestroy();
     hcreate(calls.limit);
@@ -742,19 +782,6 @@ sip_calls_clear()
     pthread_mutex_unlock(&calls.lock);
 }
 
-const char *
-sip_calculate_duration(const sip_msg_t *start, const sip_msg_t *end, char *dur)
-{
-    int seconds;
-    char duration[20];
-    // Differnce in secons
-    seconds = end->pcap_header->ts.tv_sec - start->pcap_header->ts.tv_sec;
-    // Set Human readable format
-    sprintf(duration, "%d:%02d", seconds / 60, seconds % 60);
-    sprintf(dur, "%7s", duration);
-    return dur;
-}
-
 int
 sip_set_match_expression(const char *expr, int insensitive, int invert)
 {
@@ -779,7 +806,7 @@ sip_set_match_expression(const char *expr, int insensitive, int invert)
 
     // Case insensitive requested
     if (insensitive)
-        cflags |= REG_ICASE;
+    cflags |= REG_ICASE;
 
     // Check the expresion is a compilable regexp
     return regcomp(&calls.match_regex, expr, cflags) != 0;
@@ -794,7 +821,7 @@ sip_check_match_expression(const char *payload)
         return 1;
 
 #ifdef WITH_PCRE
-    switch(pcre_exec(calls.match_regex, 0, payload, strlen(payload), 0, 0, 0, 0)) {
+    switch (pcre_exec(calls.match_regex, 0, payload, strlen(payload), 0, 0, 0, 0)) {
         case PCRE_ERROR_NOMATCH:
             return 1 == calls.match_invert;
     }
@@ -806,7 +833,6 @@ sip_check_match_expression(const char *payload)
 #endif
 }
 
-
 void
 call_set_attribute(sip_call_t *call, enum sip_attr_id id, const char *fmt, ...)
 {
@@ -835,7 +861,7 @@ call_get_attribute(sip_call_t *call, enum sip_attr_id id)
         case SIP_ATTR_TOTALDUR:
             return sip_attr_get(call->attrs, id);
         default:
-            return msg_get_attribute(call_get_next_msg(call, NULL), id);
+            return msg_get_attribute(vector_first(call->msgs), id);
     }
 
     return NULL;
@@ -871,7 +897,7 @@ sip_check_msg_ignore(sip_msg_t *msg)
     sip_attr_hdr_t *header;
 
     // Check if an ignore option exists
-    for (i = 0; i < SIP_ATTR_SENTINEL; i++) {
+    for (i = 0; i < SIP_ATTR_COUNT; i++) {
         header = sip_attr_get_header(i);
         if (is_ignored_value(header->name, msg_get_attribute(msg, header->id))) {
             return 1;
@@ -883,37 +909,37 @@ sip_check_msg_ignore(sip_msg_t *msg)
 const char *
 sip_method_str(enum sip_methods method)
 {
-    switch(method) {
-    case SIP_METHOD_REGISTER:
-        return "REGISTER";
-    case SIP_METHOD_INVITE:
-        return "INVITE";
-    case SIP_METHOD_SUBSCRIBE:
-        return "SUBSCRIBE";
-    case SIP_METHOD_NOTIFY:
-        return "NOTIFY";
-    case SIP_METHOD_OPTIONS:
-        return "OPTIONS";
-    case SIP_METHOD_PUBLISH:
-        return "PUBLISH";
-    case SIP_METHOD_MESSAGE:
-        return "MESSAGE";
-    case SIP_METHOD_CANCEL:
-        return "CANCEL";
-    case SIP_METHOD_BYE:
-        return "BYE";
-    case SIP_METHOD_ACK:
-        return "ACK";
-    case SIP_METHOD_PRACK:
-        return "PRACK";
-    case SIP_METHOD_INFO:
-        return "INFO";
-    case SIP_METHOD_REFER:
-        return "REFER";
-    case SIP_METHOD_UPDATE:
-        return "UPDATE";
-    case SIP_METHOD_SENTINEL:
-        return "";
+    switch (method) {
+        case SIP_METHOD_REGISTER:
+            return "REGISTER";
+        case SIP_METHOD_INVITE:
+            return "INVITE";
+        case SIP_METHOD_SUBSCRIBE:
+            return "SUBSCRIBE";
+        case SIP_METHOD_NOTIFY:
+            return "NOTIFY";
+        case SIP_METHOD_OPTIONS:
+            return "OPTIONS";
+        case SIP_METHOD_PUBLISH:
+            return "PUBLISH";
+        case SIP_METHOD_MESSAGE:
+            return "MESSAGE";
+        case SIP_METHOD_CANCEL:
+            return "CANCEL";
+        case SIP_METHOD_BYE:
+            return "BYE";
+        case SIP_METHOD_ACK:
+            return "ACK";
+        case SIP_METHOD_PRACK:
+            return "PRACK";
+        case SIP_METHOD_INFO:
+            return "INFO";
+        case SIP_METHOD_REFER:
+            return "REFER";
+        case SIP_METHOD_UPDATE:
+            return "UPDATE";
+        case SIP_METHOD_SENTINEL:
+            return "";
     }
     return NULL;
 }
@@ -927,3 +953,34 @@ sip_method_from_str(const char *method)
             return i;
     return atoi(method);
 }
+
+const char *
+sip_address_format(const char *address)
+{
+
+    // Return address formatted depending on active settings
+    if (setting_enabled(SETTING_DISPLAY_ALIAS)) {
+        return get_alias_value(address);
+    } else if (setting_enabled(SETTING_DISPLAY_ALIAS)) {
+        return lookup_hostname(address);
+    } else {
+        return address;
+    }
+}
+
+const char *
+sip_address_port_format(const char *addrport)
+{
+    static char aport[50];
+    char address[50];
+    int port;
+
+    strncpy(aport, addrport, 50);
+    if (sscanf(aport, "%[^:]:%d", address, &port) == 2) {
+        sprintf(aport, "%s:%d", sip_address_format(address), port);
+    }
+
+    return aport;
+}
+
+
