@@ -33,15 +33,15 @@
 #define __SNGREP_SIP_H
 
 #include "config.h"
-#include <pcap.h>
-#include <sys/time.h>
-#include <pthread.h>
-#include <arpa/inet.h>
+#include <regex.h>
 #ifdef WITH_PCRE
 #include <pcre.h>
 #endif
-#include <regex.h>
+#include "util.h"
 #include "sip_attr.h"
+#include "rtp.h"
+#include "media.h"
+#include "vector.h"
 
 //! SIP Methods
 enum sip_methods {
@@ -86,13 +86,13 @@ typedef struct sip_call_list sip_call_list_t;
  */
 struct sip_msg {
     //! Message attribute list
-    char *attrs[SIP_ATTR_SENTINEL];
+    char *attrs[SIP_ATTR_COUNT];
     //! Source address
-    char src[50];
+    char src[ADDRESSLEN];
     //! Source port
     u_short sport;
     //! Destination address
-    char dst[50];
+    char dst[ADDRESSLEN];
     //! Destination port
     u_short dport;
     //! Temporal payload data before being parsed
@@ -105,21 +105,17 @@ struct sip_msg {
     int reqresp;
     //! This message contains sdp data
     int sdp;
-    //! Message RTP position indicator
-    int rtp_pos;
-    //! Message RTP packet count
-    int rtp_count;
     //! Message Cseq
     int cseq;
+    //! SDP payload information (sdp_media_t *)
+    vector_t *medias;
+
     //! PCAP Packet Header data
     struct pcap_pkthdr *pcap_header;
     //! PCAP Packet data
     u_char *pcap_packet;
     //! Message owner
     sip_call_t *call;
-
-    //! Messages linked list
-    sip_msg_t *next;
 };
 
 /**
@@ -131,19 +127,17 @@ struct sip_msg {
  */
 struct sip_call {
     //! Call attribute list
-    char *attrs[SIP_ATTR_SENTINEL];
+    char *attrs[SIP_ATTR_COUNT];
     //! Flag this call as filtered so won't be displayed
     int filtered;
-    //! List of messages of this call
-    sip_msg_t *msgs;
-    //! Pointer to the last added message
-    sip_msg_t *last_msg;
-    //! How many messages has this call
-    int msgcnt;
+    //! For call dialogs, mark if call has not yet finished
+    int active;
+    //! List of messages of this call (sip_msg_t*)
+    vector_t *msgs;
     //! Message when conversation started
     sip_msg_t *cstart_msg;
-    //! Calls double linked list
-    sip_call_t *next, *prev;
+    //! RTP streams for this call (rtp_stream_t *)
+    vector_t *streams;
 };
 
 /**
@@ -152,11 +146,9 @@ struct sip_call {
  * This structure acts as header of calls list
  */
 struct sip_call_list {
-    // First and Last calls of the list
-    sip_call_t *first;
-    sip_call_t *last;
-    // Call counter
-    int count;
+    //! List of all captured calls
+    vector_t *list;
+
     // Max call limit
     int limit;
     //! Only store dialogs starting with INVITE
@@ -183,6 +175,7 @@ struct sip_call_list {
     regex_t reg_cseq;
     regex_t reg_from;
     regex_t reg_to;
+    regex_t reg_sdp;
     regex_t reg_sdp_addr;
     regex_t reg_sdp_port;
 
@@ -285,6 +278,21 @@ sip_load_message(const struct pcap_pkthdr *header, const char *src, u_short spor
 int
 sip_calls_count();
 
+/**
+ * @brief Return an iterator of call list
+ */
+vector_iter_t
+sip_calls_iterator();
+
+/**
+ * @brief Return stats from call list
+ *
+ * @param total Total calls processed
+ * @param displayed number of calls matching filters
+ */
+void
+sip_calls_stats(int *total, int *displayed);
+
 /**
  * @brief Append message to the call's message list
  *
@@ -332,6 +340,18 @@ call_find_by_xcallid(const char *xcallid);
 int
 call_msg_count(sip_call_t *call);
 
+/**
+ * @brief Getter for media of given messages
+ *
+ * Return the number of media structures of given msg
+ * stored in this call.
+ *
+ * @param msg SIP message structure
+ * @return how many media structures are in the msg
+ */
+int
+msg_media_count(sip_msg_t *msg);
+
 /**
  * @brief Finds the other leg of this call.
  *
@@ -345,76 +365,12 @@ call_msg_count(sip_call_t *call);
 sip_call_t *
 call_get_xcall(sip_call_t *call);
 
-/**
- * @brief Finds the next msg in a call.
- *
- * If the passed msg is NULL it returns the first message
- * in the call
- *
- * @param call SIP call structure
- * @param msg Actual SIP msg from the call (can be NULL)
- * @return Next chronological message in the call
- */
-sip_msg_t *
-call_get_next_msg(sip_call_t *call, sip_msg_t *msg);
 
-/**
- * @brief Finds the prev msg in a call.
- *
- * If the passed msg is the first message in the call
- * this function will return NULL
- *
- * @param call SIP call structure
- * @param msg Actual SIP msg from the call
- * @return Previous chronological message in the call
- */
-sip_msg_t *
-call_get_prev_msg(sip_call_t *call, sip_msg_t *msg);
+int
+call_is_active(void *item);
 
-/**
- * @brief Get next call
- *
- * General getter for call list. Never access calls list
- * directly, use this instead.
- *
- * @param cur Current call. Pass NULL to get the first call.
- * @return Next call in the list or NULL if there is no next call
- */
-sip_call_t *
-call_get_next(sip_call_t *cur);
-
-/**
- * @brief Get previous call
- *
- * General getter for call list. Never access calls list
- * directly, use this instead.
- *
- * @param cur Current call
- * @return Prev call in the list or NULL if there is no previous call
- */
-sip_call_t *
-call_get_prev(sip_call_t *cur);
-
-/**
- * @brief Get next call after applying filters and ignores
- *
- * @param cur Current call. Pass NULL to get the first call.
- * @return Next call in the list or NULL if there is no next call
- */
-sip_call_t *
-call_get_next_filtered(sip_call_t *cur);
-
-/**
- * @brief Get previous call applying filters and ignores
- *
- * General getter for call list. Never access calls list
- * directly, use this instead.
- *
- * @param cur Current call
- * @return Prev call in the list or NULL if there is no previous call
- */
-sip_call_t *
-call_get_prev_filtered(sip_call_t *cur);
+int
+msg_has_sdp(void *item);
 
 /**
  * @brief Update Call State attribute with its last parsed message
@@ -425,6 +381,15 @@ call_get_prev_filtered(sip_call_t *cur);
 void
 call_update_state(sip_call_t *call, sip_msg_t *msg);
 
+/**
+ * @brief Add a media structure to a msg
+ *
+ * @param cmsg SIP Message to be updated
+ * @param media Media structure to be added
+ */
+void
+msg_add_media(sip_msg_t *msg, sdp_media_t *media);
+
 /**
  * @brief Get message Request/Response code
  *
@@ -448,6 +413,18 @@ msg_get_reqresp(sip_msg_t *msg);
 int
 msg_parse_payload(sip_msg_t *msg, const char *payload);
 
+
+/**
+ * @brief Parse SIP Message payload for SDP media streams
+ *
+ * Parse the payload content to get SDP information
+ *
+ * @param msg SIP message structure
+ * @return 0 in all cases
+ */
+void
+msg_parse_media(sip_msg_t *msg);
+
 /**
  * @brief Check if a package is a retransmission
  *
@@ -482,6 +459,15 @@ msg_is_request(sip_msg_t *msg);
 char *
 msg_get_header(sip_msg_t *msg, char *out);
 
+/**
+ * @brief Get Time of message from packet header
+ *
+ * @param msg SIP message
+ * @return timeval structure with message first packet time
+ */
+struct timeval
+msg_get_time(sip_msg_t *msg);
+
 /**
  * @brief Remove al calls
  *
@@ -491,16 +477,6 @@ msg_get_header(sip_msg_t *msg, char *out);
 void
 sip_calls_clear();
 
-/**
- * @brief Calculate the time difference between two messages
- *
- * @param start First cronological message
- * @param start Second cronological message
- * @return Human readable time difference in MMM:SS format
- */
-const char *
-sip_calculate_duration(const sip_msg_t *start, const sip_msg_t *end, char *dur);
-
 /**
  * @brief Set Capture Matching expression
  *
@@ -603,4 +579,29 @@ sip_method_str(enum sip_methods method);
 int
 sip_method_from_str(const char *method);
 
+/**
+ * @brief Return address formatted depending on active settings
+ *
+ * Addresses can be printed in many formats depending on active settings.
+ * Alias, resolving or just printing address as is will be
+ *
+ * @param address Address in string format
+ * @return address formatted
+ */
+const char *
+sip_address_format(const char *address);
+
+/**
+ * @brief Return address:port formatted depending on active settings
+ *
+ * Addresses can be printed in many formats depending on active settings.
+ * Alias, resolving or just printing address as is will be
+ *
+ * @param addrport Address:Port in string format
+ * @return address:port formatted
+ */
+const char *
+sip_address_port_format(const char *address);
+
+
 #endif
