@@ -28,13 +28,16 @@
  * @todo Code help screen. Please.
  *
  */
+#include "config.h"
 #include <stdlib.h>
 #include <string.h>
 #include "capture.h"
+#include "ui_manager.h"
 #include "ui_call_flow.h"
 #include "ui_call_raw.h"
 #include "ui_msg_diff.h"
-#include "option.h"
+#include "util.h"
+#include "vector.h"
 
 /***
  *
@@ -57,12 +60,9 @@
  * |     |      |      |      |   |                         |
  * |     |      |      |      |   |                         |
  * |     |      |      |      |   |                         |
- * | Usefull hotkeys                                        |
+ * | Useful hotkeys                                         |
  * +--------------------------------------------------------+
  *
- * Some values we have in mind, stored in the info structure of the panel:
- *
- *  - raw_width: this represents the raw message preview width if visible.
  */
 
 /**
@@ -106,12 +106,6 @@ call_flow_create()
     return panel;
 }
 
-call_flow_info_t *
-call_flow_info(PANEL *panel)
-{
-    return (call_flow_info_t*) panel_userptr(panel);
-}
-
 void
 call_flow_destroy(PANEL *panel)
 {
@@ -125,16 +119,20 @@ call_flow_destroy(PANEL *panel)
     }
     // Delete panel window
     delwin(panel_window(panel));
-    // Delete panel
-    del_panel(panel);
+}
+
+call_flow_info_t *
+call_flow_info(PANEL *panel)
+{
+    return (call_flow_info_t*) panel_userptr(panel);
 }
 
 int
 call_flow_draw(PANEL *panel)
 {
     call_flow_info_t *info;
-    sip_msg_t *msg;
     WINDOW *win;
+    call_flow_arrow_t *arrow = NULL;
     int height, width, cline = 0;
     char title[256];
 
@@ -147,22 +145,20 @@ call_flow_draw(PANEL *panel)
     werase(win);
 
     // Set title
-    if (info->group->callcnt == 1) {
+    if (call_group_count(info->group) == 1) {
         sprintf(title, "Call flow for %s",
-                call_get_attribute(*info->group->calls, SIP_ATTR_CALLID));
+                call_get_attribute(vector_first(info->group->calls), SIP_ATTR_CALLID));
     } else {
-        sprintf(title, "Call flow for %d dialogs", info->group->callcnt);
+        sprintf(title, "Call flow for %d dialogs", call_group_count(info->group));
     }
 
     // Print color mode in title
-    if (is_option_enabled("color")) {
-        if (is_option_enabled("color.request"))
-            strcat(title, " (Color by Request/Response)");
-        if (is_option_enabled("color.callid"))
-            strcat(title, " (Color by Call-Id)");
-        if (is_option_enabled("color.cseq"))
-            strcat(title, " (Color by CSeq)");
-    }
+    if (setting_has_value(SETTING_COLORMODE, "request"))
+        strcat(title, " (Color by Request/Response)");
+    if (setting_has_value(SETTING_COLORMODE, "callid"))
+        strcat(title, " (Color by Call-Id)");
+    if (setting_has_value(SETTING_COLORMODE, "cseq"))
+        strcat(title, " (Color by CSeq)");
 
     // Draw panel title
     draw_title(panel, title);
@@ -173,22 +169,25 @@ call_flow_draw(PANEL *panel)
     // Redraw columns
     call_flow_draw_columns(panel);
 
-    // Let's start from the first displayed message (not the first in the call group)
-    for (msg = info->first_msg; msg; msg = call_group_get_next_msg(info->group, msg)) {
-        // Draw messages until the Message height has been filled
-        if (call_flow_draw_message(panel, msg, cline) != 0)
-            break;
-        // One message fills 2 lines
-        cline += 2;
+    for (arrow = info->first_arrow; arrow; arrow = call_flow_next_arrow(panel, arrow)) {
+        if (arrow->type == CF_ARROW_SIP) {
+            if (!call_flow_draw_message(panel, arrow, cline))
+                break;
+        } else if (arrow->type == CF_ARROW_RTP) {
+            if (!call_flow_draw_stream(panel, arrow, cline))
+                break;
+        }
+        cline += arrow->height;
     }
 
     // If there are only three columns, then draw the raw message on this panel
-    if (is_option_enabled("cf.forceraw")) {
-        call_flow_draw_raw(panel, info->cur_msg);
+    if (setting_enabled(SETTING_CF_FORCERAW)) {
+        call_flow_draw_raw(panel, call_flow_arrow_message(info->cur_arrow));
     }
 
     // Draw the scrollbar
-    draw_vscrollbar(info->flow_win, call_group_msg_number(info->group, info->first_msg) * 2,
+    draw_vscrollbar(info->flow_win,
+                    call_group_msg_number(info->group, call_flow_arrow_message(info->first_arrow)) * 2,
                     call_group_msg_count(info->group) * 2, 1);
 
     // Redraw flow win
@@ -203,11 +202,11 @@ call_flow_draw_footer(PANEL *panel)
 {
     const char *keybindings[] = {
         key_action_key_str(ACTION_PREV_SCREEN), "Calls List",
-        key_action_key_str(ACTION_CONFIRM), "Raw Message",
+        key_action_key_str(ACTION_CONFIRM), "Raw",
         key_action_key_str(ACTION_SELECT), "Compare",
         key_action_key_str(ACTION_SHOW_HELP), "Help",
-        key_action_key_str(ACTION_SDP_INFO), "SDP mode",
-        key_action_key_str(ACTION_TOGGLE_RAW), "Toggle Raw",
+        key_action_key_str(ACTION_SDP_INFO), "SDP",
+        key_action_key_str(ACTION_TOGGLE_MEDIA), "RTP",
         key_action_key_str(ACTION_SHOW_FLOW_EX), "Extended",
         key_action_key_str(ACTION_COMPRESS), "Compressed",
         key_action_key_str(ACTION_SHOW_RAW), "Raw",
@@ -223,8 +222,11 @@ call_flow_draw_columns(PANEL *panel)
 {
     call_flow_info_t *info;
     call_flow_column_t *column;
+    sip_call_t *call = NULL;
+    rtp_stream_t *stream;
     WINDOW *win;
     sip_msg_t *msg;
+    vector_iter_t streams;
     int flow_height, flow_width;
     const char *coltext;
     char address[50], *end;
@@ -238,8 +240,21 @@ call_flow_draw_columns(PANEL *panel)
     // Load columns
     for (msg = call_group_get_next_msg(info->group, NULL); msg;
          msg = call_group_get_next_msg(info->group, msg)) {
-        call_flow_column_add(panel, CALLID(msg), SRC(msg), SRCHOST(msg));
-        call_flow_column_add(panel, CALLID(msg), DST(msg), DSTHOST(msg));
+        call_flow_column_add(panel, CALLID(msg), SRC(msg));
+        call_flow_column_add(panel, CALLID(msg), DST(msg));
+    }
+
+    // Add RTP columns FIXME Really
+    if (setting_enabled(SETTING_CF_MEDIA)) {
+        while ((call = call_group_get_next(info->group, call)) ) {
+            streams = vector_iterator(call->streams);
+            while ((stream = vector_iterator_next(&streams))) {
+                if (stream_get_count(stream)) {
+                    call_flow_column_add(panel, NULL, stream->ip_src);
+                    call_flow_column_add(panel, NULL, stream->ip_dst);
+                }
+            }
+        }
     }
 
     // Draw vertical columns lines
@@ -252,10 +267,10 @@ call_flow_draw_columns(PANEL *panel)
         strcpy(address, column->addr);
         if ((end = strchr(address, ':')))
             *end = '\0';
-        if (is_local_address_str(address) && is_option_enabled("cf.localhighlight"))
+        if (is_local_address_str(address) && setting_enabled(SETTING_CF_LOCALHIGHLIGHT))
             wattron(win, A_BOLD);
 
-        coltext = (is_option_enabled("sngrep.displayhost")) ? column->host : column->addr;
+        coltext = sip_address_port_format(column->addr);
         mvwprintw(win, 2, 10 + 30 * column->colpos + (22 - strlen(coltext)) / 2, "%s", coltext);
         wattroff(win, A_BOLD);
     }
@@ -263,21 +278,23 @@ call_flow_draw_columns(PANEL *panel)
     return 0;
 }
 
-int
-call_flow_draw_message(PANEL *panel, sip_msg_t *msg, int cline)
+call_flow_arrow_t *
+call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
 {
     call_flow_info_t *info;
     WINDOW *win;
+    sdp_media_t *media;
     const char *msg_time;
     const char *msg_callid;
     const char *msg_method;
-    const char *msg_from;
-    const char *msg_to;
     const char *msg_src;
     const char *msg_dst;
     char method[80];
+    char delta[15] = {};
     int height, width;
-    int rtp_count, sdp_port, sdp_local;
+    char mediastr[40];
+    sip_msg_t *msg = arrow->msg;
+    vector_iter_t medias;
 
     // Get panel information
     info = call_flow_info(panel);
@@ -285,41 +302,65 @@ call_flow_draw_message(PANEL *panel, sip_msg_t *msg, int cline)
     win = info->flow_win;
     getmaxyx(win, height, width);
 
-    // Get the current line in the win
-    if (cline > height + 2)
-        return 1;
+    // Store arrow start line
+    arrow->line = cline;
+
+    // Calculate how many lines this message requires
+    arrow->height = call_flow_arrow_height(panel, arrow);
+
+    // Check this message fits on the panel
+    if (cline > height + arrow->height)
+        return NULL;
 
     // Get message attributes
     msg_time = msg_get_attribute(msg, SIP_ATTR_TIME);
     msg_callid = msg_get_attribute(msg, SIP_ATTR_CALLID);
     msg_method = msg_get_attribute(msg, SIP_ATTR_METHOD);
-    msg_from = msg_get_attribute(msg, SIP_ATTR_SIPFROM);
-    msg_to = msg_get_attribute(msg, SIP_ATTR_SIPTO);
     msg_src = msg_get_attribute(msg, SIP_ATTR_SRC);
     msg_dst = msg_get_attribute(msg, SIP_ATTR_DST);
 
+    // Check method has value. We prefer empty arrows rather than crashes
+    if (!msg_method)
+        msg_method = "";
+
     // Print timestamp
+    if (info->selected == arrow)
+        wattron(win, COLOR_PAIR(CP_CYAN_ON_DEF));
     mvwprintw(win, cline, 2, "%s", msg_time);
 
+    // Print delta from selected message
+    if (!info->selected) {
+        if (setting_enabled(SETTING_CF_DELTA))
+            timeval_to_delta(msg_get_time(msg), msg_get_time(call_group_get_next_msg(info->group, msg)), delta);
+    } else if (info->cur_arrow == arrow) {
+        timeval_to_delta(msg_get_time(call_flow_arrow_message(info->selected)), msg_get_time(msg), delta);
+    }
+
+    if (strlen(delta)) {
+        wattron(win, COLOR_PAIR(CP_CYAN_ON_DEF));
+        mvwprintw(win, cline + 1 , 2, "%15s", delta);
+    }
+    wattroff(win, COLOR_PAIR(CP_CYAN_ON_DEF));
+
     // Get Message method (include extra info)
-    memset(method, 0, sizeof(method));
-    if (msg_method)
-        sprintf(method, "%s", msg_method);
+    sprintf(method, "%s", msg_method);
 
     // If message has sdp information
-    if (msg->sdp) {
-        // Change method text if sdpinfo mode is requested
-        if (is_option_enabled("cf.sdpinfo")) {
-            // Show message sdp in title
-            memset(method, 0, sizeof(method));
-            strncpy(method, msg_method, 3);
-            sprintf(method + strlen(method), " (%s:%s)",
-                    msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS),
-                    msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
-        } else {
-            // Show sdp tag in tittle
-            strcat(method, " (SDP)");
-        }
+    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "off")) {
+        // Show sdp tag in tittle
+        sprintf(method, "%s (SDP)", msg_method);
+    }
+
+    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "first")) {
+        sprintf(method, "%.3s (%s:%s)",
+                msg_method,
+                msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS),
+                msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
+    }
+    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
+        sprintf(method, "%.3s (%s)",
+                msg_method,
+                msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS));
     }
 
     // Draw message type or status and line
@@ -343,31 +384,27 @@ call_flow_draw_message(PANEL *panel, sip_msg_t *msg, int cline)
     int distance = abs(endpos - startpos) - 3;
 
     // Highlight current message
-    if (msg == info->cur_msg) {
-        if (is_option_value("cf.highlight", "reverse")) {
+    if (arrow == info->cur_arrow) {
+        if (setting_has_value(SETTING_CF_HIGHTLIGHT, "reverse")) {
             wattron(win, A_REVERSE);
         }
-        if (is_option_value("cf.highlight", "bold")) {
+        if (setting_has_value(SETTING_CF_HIGHTLIGHT, "bold")) {
             wattron(win, A_BOLD);
         }
-        if (is_option_value("cf.highlight", "reversebold")) {
+        if (setting_has_value(SETTING_CF_HIGHTLIGHT, "reversebold")) {
             wattron(win, A_REVERSE);
             wattron(win, A_BOLD);
         }
     }
 
     // Color the message {
-    if (is_option_enabled("color.request")) {
-        // Determine arrow color
-        if (msg_is_request(msg)) {
-            msg->color = CP_RED_ON_DEF;
-        } else {
-            msg->color = CP_GREEN_ON_DEF;
-        }
-    } else if (is_option_enabled("color.callid")) {
+    if (setting_has_value(SETTING_COLORMODE, "request")) {
+        // Color by request / response
+        msg->color = (msg_is_request(msg)) ? CP_RED_ON_DEF : CP_GREEN_ON_DEF;
+    } else if (setting_has_value(SETTING_COLORMODE, "callid")) {
         // Color by call-id
         msg->color = call_group_color(info->group, msg->call);
-    } else if (is_option_enabled("color.cseq")) {
+    } else if (setting_has_value(SETTING_COLORMODE, "cseq")) {
         // Color by CSeq within the same call
         msg->color = msg->cseq % 7 + 1;
     }
@@ -375,48 +412,41 @@ call_flow_draw_message(PANEL *panel, sip_msg_t *msg, int cline)
     // Turn on the message color
     wattron(win, COLOR_PAIR(msg->color));
 
+    // Clear the line
     mvwprintw(win, cline, startpos + 2, "%*s", distance, "");
+    // Draw method
     mvwprintw(win, cline, startpos + distance / 2 - msglen / 2 + 2, "%.26s", method);
-    if (msg == info->selected) {
-        mvwhline(win, cline + 1, startpos + 2, '=', distance);
-    } else {
-        mvwhline(win, cline + 1, startpos + 2, ACS_HLINE, distance);
+    cline ++;
+    // Draw media information
+    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
+        medias = vector_iterator(msg->medias);
+        while ((media = vector_iterator_next(&medias))) {
+            sprintf(mediastr, "%s %d (%s)",
+                    media_get_type(media),
+                    media_get_port(media),
+                    media_get_format(media));
+            mvwprintw(win, cline++, startpos + distance / 2 - strlen(mediastr) / 2 + 2, mediastr);
+        }
     }
 
-
-    // If message has SDP check if packet count in SDP Port has changed
-    if (info->show_rtp && msg->sdp) {
-        // Check if SDP address is local
-        sdp_local = is_local_address_str(msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS));
-        // Check if packet count has changed in SDP port
-        sdp_port = atoi(msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
-        // Retrieve packet count for SDP port
-        rtp_count = capture_packet_count_port(sdp_local, sdp_port);
-
-        if (msg->rtp_count != rtp_count) {
-            msg->rtp_count = rtp_count;
-            msg->rtp_pos = (msg->rtp_pos + 1) % distance;
-
-            if (!strcasecmp(msg_src, column1->addr)) {
-                mvwaddch(win, cline + 1, startpos + distance - msg->rtp_pos, '<');
-            } else {
-                mvwaddch(win, cline + 1, startpos + 2 + msg->rtp_pos, '>');
-            }
-        }
+    if (arrow == info->selected) {
+        mvwhline(win, cline, startpos + 2, '=', distance);
+    } else {
+        mvwhline(win, cline, startpos + 2, ACS_HLINE, distance);
     }
 
     // Write the arrow at the end of the message (two arros if this is a retrans)
     if (!strcasecmp(msg_src, column1->addr)) {
-        mvwaddch(win, cline + 1, endpos - 2, '>');
+        mvwaddch(win, cline, endpos - 2, '>');
         if (msg_is_retrans(msg)) {
-            mvwaddch(win, cline + 1, endpos - 3, '>');
-            mvwaddch(win, cline + 1, endpos - 4, '>');
+            mvwaddch(win, cline, endpos - 3, '>');
+            mvwaddch(win, cline, endpos - 4, '>');
         }
     } else {
-        mvwaddch(win, cline + 1, startpos + 2, '<');
+        mvwaddch(win, cline, startpos + 2, '<');
         if (msg_is_retrans(msg)) {
-            mvwaddch(win, cline + 1, startpos + 3, '<');
-            mvwaddch(win, cline + 1, startpos + 4, '<');
+            mvwaddch(win, cline, startpos + 3, '<');
+            mvwaddch(win, cline, startpos + 4, '<');
         }
     }
 
@@ -425,35 +455,282 @@ call_flow_draw_message(PANEL *panel, sip_msg_t *msg, int cline)
     wattroff(win, COLOR_PAIR(CP_GREEN_ON_DEF));
     wattroff(win, COLOR_PAIR(CP_CYAN_ON_DEF));
     wattroff(win, COLOR_PAIR(CP_YELLOW_ON_DEF));
-    wattroff(win, A_BOLD);
-    wattroff(win, A_REVERSE);
+    wattroff(win, A_BOLD | A_REVERSE);
+
+    return arrow;
+}
+
+
+call_flow_arrow_t *
+call_flow_draw_stream(PANEL *panel, call_flow_arrow_t *arrow, int cline)
+{
+    call_flow_info_t *info;
+    WINDOW *win;
+//    char time[20];
+    char codec[50];
+    int height, width;
+    rtp_stream_t *stream = arrow->stream;
+
+    // Get panel information
+    info = call_flow_info(panel);
+    // Get the messages window
+    win = info->flow_win;
+    getmaxyx(win, height, width);
+
+    // Store arrow start line
+    arrow->line = cline;
+
+    // Calculate how many lines this message requires
+    arrow->height = call_flow_arrow_height(panel, arrow);
+
+    // Check this media fits on the panel
+    if (cline > height + arrow->height)
+        return NULL;
+
+    // Print timestamp
+    //mvwprintw(win, cline, 2, "%s", timeval_to_time(stream->time, time));
+
+    // Get Message method (include extra info)
+    sprintf(codec, "RTP (%s) %d", rtp_get_codec(stream->format, NULL), stream_get_count(stream));
+
+    // Get origin and destination column
+    call_flow_column_t *column1 = call_flow_column_get(panel, 0, stream->ip_src);
+    call_flow_column_t *column2 = call_flow_column_get(panel, 0, stream->ip_dst);
+
+    call_flow_column_t *tmp;
+    if (column1->colpos > column2->colpos) {
+        tmp = column1;
+        column1 = column2;
+        column2 = tmp;
+    }
+
+    int startpos = 20 + 30 * column1->colpos;
+    int endpos = 20 + 30 * column2->colpos;
+    int distance = abs(endpos - startpos) - 4;
+
+    // Highlight current message
+    if (arrow == info->cur_arrow) {
+        if (setting_has_value(SETTING_CF_HIGHTLIGHT, "reverse")) {
+            wattron(win, A_REVERSE);
+        }
+        if (setting_has_value(SETTING_CF_HIGHTLIGHT, "bold")) {
+            wattron(win, A_BOLD);
+        }
+        if (setting_has_value(SETTING_CF_HIGHTLIGHT, "reversebold")) {
+            wattron(win, A_REVERSE);
+            wattron(win, A_BOLD);
+        }
+    }
+
+    // Clear the line
+    mvwprintw(win, cline, startpos + 2, "%*s", distance, "");
+    // Draw method
+    mvwprintw(win, cline, startpos + (distance) / 2 - strlen(codec) / 2 + 2, "%s", codec);
+    cline ++;
+    // Draw line between columns
+    mvwhline(win, cline, startpos + 2, ACS_HLINE, distance);
+    // Write the arrow at the end of the message (two arros if this is a retrans)
+    if (call_flow_column_get(panel, 0, stream->ip_src) == column1) {
+        mvwprintw(win, cline, startpos - 5, "%d", stream->sport);
+        mvwprintw(win, cline, endpos + 1, "%d", stream->dport);
+        if (distance > 0)
+            mvwaddch(win, cline, endpos - 2, '>');
+        else
+            mvwaddch(win, cline, endpos, '>');
+    } else {
+        mvwprintw(win, cline, endpos  + 1, "%d", stream->sport);
+        mvwprintw(win, cline, startpos - 5, "%d", stream->dport);
+        if (distance > 0)
+            mvwaddch(win, cline, startpos + 2, '<');
+        else
+            mvwaddch(win, cline, startpos, '<');
+    }
+
+    wattroff(win, A_BOLD | A_REVERSE);
+
+    return arrow;
+}
+
+call_flow_arrow_t *
+call_flow_next_arrow(PANEL *panel, const call_flow_arrow_t *cur)
+{
+    sip_msg_t *msg = NULL;
+    rtp_stream_t *stream = NULL;
+    struct timeval cur_time;
+    call_flow_info_t *info;
+    call_flow_arrow_t *next, *tmp;
+
+    // Get panel information
+    info = call_flow_info(panel);
+
+    if (!cur) {
+        memset(&cur_time, 0, sizeof(struct timeval));
+    } else if (cur->type == CF_ARROW_SIP) {
+        cur_time = cur->msg->pcap_header->ts;
+    } else if (cur->type == CF_ARROW_RTP) {
+        cur_time = cur->stream->time;
+    }
+
+    // Look for the next message
+    while ((msg = call_group_get_next_msg(info->group, msg))) {
+        if (timeval_is_older(msg->pcap_header->ts, cur_time)) {
+            break;
+        }
+    }
+
+    if (setting_enabled(SETTING_CF_MEDIA)) {
+        // Look for the next stream
+        while ((stream = call_group_get_next_stream(info->group, stream))) {
+            if (timeval_is_older(stream->time, cur_time)) {
+                break;
+            }
+        }
+    }
+
+    if (!msg && !stream)
+        return NULL;  /* Nothing goes next */
+
+    /* a rtp stream goes next */
+    if (!msg) {
+        if ((next = call_flow_arrow_find(panel, stream)))
+            return next;
+        // Create a new arrow to store next info
+        next = malloc(sizeof(call_flow_arrow_t));
+        memset(next, 0, sizeof(call_flow_arrow_t));
+        next->type = CF_ARROW_RTP;
+        next->stream = stream;
+    } else if (!stream) {
+        /* a sip message goes next */
+        if ((next = call_flow_arrow_find(panel, msg)))
+            return next;
+        // Create a new arrow to store next info
+        next = malloc(sizeof(call_flow_arrow_t));
+        memset(next, 0, sizeof(call_flow_arrow_t));
+        next->type = CF_ARROW_SIP;
+        next->msg = msg;
+    } else {
+        /* Determine what goes next */
+        if (timeval_is_older(msg->pcap_header->ts, stream->time)) {
+            if ((next = call_flow_arrow_find(panel, stream)))
+                return next;
+            // Create a new arrow to store next info
+            next = malloc(sizeof(call_flow_arrow_t));
+            memset(next, 0, sizeof(call_flow_arrow_t));
+            next->type = CF_ARROW_RTP;
+            next->stream = stream;
+        } else {
+            if ((next = call_flow_arrow_find(panel, msg)))
+                return next;
+            // Create a new arrow to store next info
+            next = malloc(sizeof(call_flow_arrow_t));
+            memset(next, 0, sizeof(call_flow_arrow_t));
+            next->type = CF_ARROW_SIP;
+            next->msg = msg;
+        }
+    }
+
+    // Add this arrow to the list and return it
+    if (!info->arrows) {
+        info->arrows = next;
+    } else {
+        for (tmp = info->arrows; tmp->next; tmp = tmp->next);
+        tmp->next = next;
+    }
+    return next;
+}
+
+call_flow_arrow_t *
+call_flow_prev_arrow(PANEL *panel, const call_flow_arrow_t *cur)
+{
+    call_flow_arrow_t *arrow = NULL, *prev = NULL;
+    while ((arrow = call_flow_next_arrow(panel, arrow))) {
+        if (arrow == cur) {
+            return prev;
+        }
+        prev = arrow;
+    }
+    return NULL;
+}
+
+int
+call_flow_arrow_height(PANEL *panel, const call_flow_arrow_t *arrow)
+{
+    if (arrow->type == CF_ARROW_SIP) {
+        if (!arrow->msg->sdp)
+            return 2;
+        if (setting_has_value(SETTING_CF_SDP_INFO, "off"))
+            return 2;
+        if (setting_has_value(SETTING_CF_SDP_INFO, "first"))
+            return 2;
+        if (setting_has_value(SETTING_CF_SDP_INFO, "full"))
+            return msg_media_count(arrow->msg) + 2;
+    } else if (arrow->type == CF_ARROW_RTP) {
+        return 2;
+    }
 
     return 0;
 }
 
+call_flow_arrow_t *
+call_flow_arrow_find(PANEL *panel, const void *data)
+{
+    call_flow_info_t *info;
+    call_flow_arrow_t *arrow;
+
+    if (!data)
+        return NULL;
+
+    if (!(info = call_flow_info(panel)))
+        return NULL;
+
+    for (arrow = info->arrows; arrow; arrow = arrow->next)
+        if (arrow->msg == data || arrow->stream == data)
+            return arrow;
+
+    return arrow;
+}
+
+sip_msg_t *
+call_flow_arrow_message(const  call_flow_arrow_t *arrow)
+{
+    if (!arrow)
+        return NULL;
+    if (arrow->type == CF_ARROW_SIP)
+        return arrow->msg;
+    if (arrow->type == CF_ARROW_RTP)
+        return arrow->stream->media->msg;
+    return NULL;
+}
+
 int
 call_flow_draw_raw(PANEL *panel, sip_msg_t *msg)
 {
     call_flow_info_t *info;
     WINDOW *win, *raw_win;
     int raw_width, raw_height, height, width;
+    int min_raw_width, fixed_raw_width;
 
     // Get panel information
-    info = call_flow_info(panel);
+    if (!(info = call_flow_info(panel)))
+            return 1;
 
     // Get window of main panel
     win = panel_window(panel);
     getmaxyx(win, height, width);
 
+    // Get min raw width
+    min_raw_width = setting_get_intvalue(SETTING_CF_RAWMINWIDTH);
+    fixed_raw_width = setting_get_intvalue(SETTING_CF_RAWFIXEDWIDTH);
+
     // Calculate the raw data width (width - used columns for flow - vertical lines)
     raw_width = width - (31 + 30 * info->columns->colpos) - 2;
     // We can define a mininum size for rawminwidth
-    if (raw_width < get_option_int_value("cf.rawminwidth")) {
-        raw_width = get_option_int_value("cf.rawminwidth");
+    if (raw_width < min_raw_width) {
+        raw_width = min_raw_width;
     }
     // We can configure an exact raw size
-    if (get_option_int_value("cf.rawfixedwidth") != -1) {
-        raw_width = get_option_int_value("cf.rawfixedwidth");
+    if (fixed_raw_width > 0) {
+        raw_width = fixed_raw_width;
     }
 
     // Height of raw window is always available size minus 6 lines for header/footer
@@ -495,10 +772,10 @@ call_flow_handle_key(PANEL *panel, int key)
 {
     int i, raw_width, height, width;
     call_flow_info_t *info = call_flow_info(panel);
-    sip_msg_t *next = NULL, *prev = NULL;
+    call_flow_arrow_t *next, *prev;
     ui_t *next_panel;
     sip_call_group_t *group;
-    int rnpag_steps = get_option_int_value("cf.scrollstep");
+    int rnpag_steps = setting_get_intvalue(SETTING_CF_SCROLLSTEP);
     int action = -1;
 
     // Sanity check, this should not happen
@@ -513,33 +790,27 @@ call_flow_handle_key(PANEL *panel, int key)
         switch(action) {
             case ACTION_DOWN:
                 // Check if there is a call below us
-                if (!(next = call_group_get_next_msg(info->group, info->cur_msg)))
+                if (!(next = call_flow_next_arrow(panel, info->cur_arrow)))
                     break;
-                info->cur_msg = next;
-                info->cur_line += 2;
+                info->cur_line += call_flow_arrow_height(panel, info->cur_arrow);
+
                 // If we are out of the bottom of the displayed list
                 // refresh it starting in the next call
                 if (info->cur_line >= height) {
-                    info->first_msg = call_group_get_next_msg(info->group, info->first_msg);
-                    info->cur_line -= 2;
+                    info->cur_line -= call_flow_arrow_height(panel, info->first_arrow);
+                    info->first_arrow = call_flow_next_arrow(panel, info->first_arrow);
                 }
+                info->cur_arrow = next;
                 break;
             case ACTION_UP:
-                // Loop through messages until we found current message
-                // storing the previous one
-                while ((next = call_group_get_next_msg(info->group, next))) {
-                    if (next == info->cur_msg)
-                        break;
-                    prev = next;
-                }
-                // We're at the first message already
-                if (!prev)
+                // Get previous message
+                if (!(prev = call_flow_prev_arrow(panel, info->cur_arrow)))
                     break;
-                info->cur_msg = prev;
-                info->cur_line -= 2;
+                info->cur_line -= call_flow_arrow_height(panel, info->cur_arrow);
+                info->cur_arrow = prev;
                 if (info->cur_line <= 0) {
-                    info->first_msg = info->cur_msg;
-                    info->cur_line += 2;
+                    info->cur_line += call_flow_arrow_height(panel, prev);
+                    info->first_arrow = prev;
                 }
                 break;
             case ACTION_HNPAGE:
@@ -558,82 +829,87 @@ call_flow_handle_key(PANEL *panel, int key)
                 for (i = 0; i < rnpag_steps; i++)
                     call_flow_handle_key(panel, KEY_UP);
                 break;
-            case ACTION_TOGGLE_RTP:
-                 info->show_rtp = (info->show_rtp)?0:1;
-                 break;
+            case ACTION_BEGIN:
+                call_flow_set_group(info->group);
+                break;
+            case ACTION_END:
+                call_flow_set_group(info->group);
+                for (i=0; i < call_group_msg_count(info->group); i++)
+                    call_flow_handle_key(panel, KEY_DOWN);
+                break;
             case ACTION_SHOW_FLOW_EX:
                 werase(panel_window(panel));
                 // KEY_X , Display current call flow
-                if (info->group->callcnt == 1) {
-                    group = call_group_create();
-                    call_group_add(group, info->group->calls[0]);
-                    call_group_add(group, call_get_xcall(info->group->calls[0]));
-                    call_flow_set_group(group);
-                } else {
-                    group = call_group_create();
-                    call_group_add(group, info->group->calls[0]);
-                    call_flow_set_group(group);
+                group = call_group_create();
+                if (call_group_count(info->group) == 1) {
+                    call_group_add(group, call_get_xcall(vector_first(info->group->calls)));
                 }
+                call_group_add(group, vector_first(info->group->calls));
+                call_flow_set_group(group);
                 break;
             case ACTION_SHOW_RAW:
                 // KEY_R, display current call in raw mode
-                next_panel = ui_create(ui_find_by_type(PANEL_CALL_RAW));
-                // TODO
+                ui_create_panel(PANEL_CALL_RAW);
                 call_raw_set_group(info->group);
-                wait_for_input(next_panel);
-                break;
-            case ACTION_INCREASE_RAW:
-                raw_width = getmaxx(info->raw_win);
-                if (raw_width - 2 > 1) {
-                    set_option_int_value("cf.rawfixedwidth", raw_width - 2);
-                }
                 break;
             case ACTION_DECREASE_RAW:
+                raw_width = getmaxx(info->raw_win);
+                if (raw_width - 2 > 1) {
+                    setting_set_intvalue(SETTING_CF_RAWFIXEDWIDTH, raw_width - 2);
+                }
+                break;
+            case ACTION_INCREASE_RAW:
                 raw_width = getmaxx(info->raw_win);
                 if (raw_width + 2 < COLS - 1) {
-                    set_option_int_value("cf.rawfixedwidth", raw_width + 2);
+                    setting_set_intvalue(SETTING_CF_RAWFIXEDWIDTH, raw_width + 2);
                 }
                 break;
             case ACTION_RESET_RAW:
-                set_option_int_value("cf.rawfixedwidth", -1);
+                setting_set_intvalue(SETTING_CF_RAWFIXEDWIDTH, -1);
                 break;
             case ACTION_ONLY_SDP:
                 // Toggle SDP mode
                 info->group->sdp_only = !(info->group->sdp_only);
+                // Disable sdp_only if there are not messages with sdp
+                if (call_group_msg_count(info->group) == 0)
+                    info->group->sdp_only = 0;
+                // Reset screen
                 call_flow_set_group(info->group);
                 break;
             case ACTION_SDP_INFO:
-                set_option_value("cf.sdpinfo", is_option_enabled("cf.sdpinfo") ? "off" : "on");
+                setting_toggle(SETTING_CF_SDP_INFO);
+                break;
+            case ACTION_TOGGLE_MEDIA:
+                setting_toggle(SETTING_CF_MEDIA);
                 break;
             case ACTION_TOGGLE_RAW:
-                set_option_value("cf.forceraw", is_option_enabled("cf.forceraw") ? "off" : "on");
+                setting_toggle(SETTING_CF_FORCERAW);
                 break;
             case ACTION_COMPRESS:
-                set_option_value("cf.splitcallid", is_option_enabled("cf.splitcallid") ? "off" : "on");
+                setting_toggle(SETTING_CF_SPLITCALLID);
                 // Force columns reload
-                info->columns = NULL;
+                call_flow_set_group(info->group);
                 break;
             case ACTION_SELECT:
                 if (!info->selected) {
-                    info->selected = info->cur_msg;
+                    info->selected = info->cur_arrow;
                 } else {
-                    if (info->selected == info->cur_msg) {
+                    if (info->selected == info->cur_arrow) {
                         info->selected = NULL;
                     } else {
                         // Show diff panel
-                        next_panel = ui_create(ui_find_by_type(PANEL_MSG_DIFF));
-                        msg_diff_set_msgs(ui_get_panel(next_panel), info->selected, info->cur_msg);
-                        wait_for_input(next_panel);
+                        next_panel = ui_create_panel(PANEL_MSG_DIFF);
+                        msg_diff_set_msgs(ui_get_panel(next_panel),
+                                          call_flow_arrow_message(info->selected),
+                                          call_flow_arrow_message(info->cur_arrow));
                     }
                 }
                 break;
             case ACTION_CONFIRM:
                 // KEY_ENTER, display current message in raw mode
-                next_panel = ui_create(ui_find_by_type(PANEL_CALL_RAW));
-                // TODO
+                ui_create_panel(PANEL_CALL_RAW);
                 call_raw_set_group(info->group);
-                call_raw_set_msg(info->cur_msg);
-                wait_for_input(next_panel);
+                call_raw_set_msg(call_flow_arrow_message(info->cur_arrow));
                 break;
             default:
                 // Parse next action
@@ -652,14 +928,12 @@ int
 call_flow_help(PANEL *panel)
 {
     WINDOW *help_win;
-    PANEL *help_panel;
     int height, width;
 
     // Create a new panel and show centered
     height = 26;
     width = 65;
     help_win = newwin(height, width, (LINES - height) / 2, (COLS - width) / 2);
-    help_panel = new_panel(help_win);
 
     // Set the window title
     mvwprintw(help_win, 1, 18, "Call Flow Help");
@@ -696,7 +970,7 @@ call_flow_help(PANEL *panel)
     mvwprintw(help_win, 12, 2, "F2/d        Toggle SDP Address:Port info");
     mvwprintw(help_win, 13, 2, "F3/t        Toggle raw preview display");
     mvwprintw(help_win, 14, 2, "F4/X        Show call-flow with X-CID/X-Call-ID dialog");
-    mvwprintw(help_win, 15, 2, "F5/S        Toggle compressed view (One address <=> one column");
+    mvwprintw(help_win, 15, 2, "F5/s        Toggle compressed view (One address <=> one column");
     mvwprintw(help_win, 16, 2, "F6/R        Show original call messages in raw mode");
     mvwprintw(help_win, 17, 2, "F7/c        Cycle between available color modes");
     mvwprintw(help_win, 18, 2, "F8/C        Turn on/off message syntax highlighting");
@@ -707,8 +981,6 @@ call_flow_help(PANEL *panel)
 
     // Press any key to close
     wgetch(help_win);
-    update_panels();
-    doupdate();
 
     return 0;
 }
@@ -726,15 +998,16 @@ call_flow_set_group(sip_call_group_t *group)
         return -1;
 
     info->group = group;
-    info->cur_msg = info->first_msg = call_group_get_next_msg(group, NULL);
+    info->cur_arrow = info->first_arrow = call_flow_next_arrow(panel, NULL);
     info->cur_line = 1;
     info->columns = NULL;
+    info->selected = NULL;
 
     return 0;
 }
 
 void
-call_flow_column_add(PANEL *panel, const char *callid, const char *addr, const char *host)
+call_flow_column_add(PANEL *panel, const char *callid, const char *addr)
 {
     call_flow_info_t *info;
     call_flow_column_t *column;
@@ -743,6 +1016,9 @@ call_flow_column_add(PANEL *panel, const char *callid, const char *addr, const c
     if (!(info = call_flow_info(panel)))
         return;
 
+    if (!addr || !strlen(addr))
+        return;
+
     if (call_flow_column_get(panel, callid, addr))
         return;
 
@@ -762,7 +1038,6 @@ call_flow_column_add(PANEL *panel, const char *callid, const char *addr, const c
     memset(column, 0, sizeof(call_flow_column_t));
     column->callid = callid;
     column->addr = addr;
-    column->host = host;
     column->colpos = colpos;
     column->next = info->columns;
     info->columns = column;
@@ -773,14 +1048,29 @@ call_flow_column_get(PANEL *panel, const char *callid, const char *addr)
 {
     call_flow_info_t *info;
     call_flow_column_t *columns;
+    int match_port;
+    char coladdr[ADDRESSLEN + 6];
+    char *dots;
 
     if (!(info = call_flow_info(panel)))
         return NULL;
 
+    // Look for address or address:port ?
+    match_port = (strchr(addr, ':') != NULL);
+
     columns = info->columns;
     while (columns) {
-        if (!strcasecmp(addr, columns->addr)) {
-            if (is_option_enabled("cf.splitcallid"))
+        // Copy address:port column label
+        strcpy(coladdr, columns->addr);
+
+        // Remove port if we want to match only address
+        if (!match_port && (dots = strchr(coladdr, ':')))
+            *dots = '\0';
+
+        if (!strcasecmp(addr, coladdr)) {
+            if (!match_port)
+                return columns;
+            if (setting_enabled(SETTING_CF_SPLITCALLID))
                 return columns;
             if (columns->callid && !strcasecmp(callid, columns->callid))
                 return columns;
