@@ -35,10 +35,12 @@
 #include "option.h"
 #include "filter.h"
 #include "capture.h"
+#include "ui_manager.h"
 #include "ui_call_list.h"
 #include "ui_call_flow.h"
 #include "ui_call_raw.h"
-#include "ui_save_pcap.h"
+#include "ui_save.h"
+#include "sip.h"
 
 /**
  * Ui Structure definition for Call List panel
@@ -74,7 +76,7 @@ call_list_create()
     set_panel_userptr(panel, (void*) info);
 
     // Add configured columns
-    for (i = 0; i < SIP_ATTR_SENTINEL; i++) {
+    for (i = 0; i < SIP_ATTR_COUNT; i++) {
         // Get column attribute name from options
         sprintf(option, "cl.column%d", i);
         if ((field = get_option_value(option))) {
@@ -110,9 +112,14 @@ call_list_create()
     info->list_win = subwin(win, height - 5, width, 4, 0);
     info->group = call_group_create();
 
+    // Get current call list
+    info->calls = sip_calls_iterator();
+    vector_iterator_set_filter(&info->calls, filter_check_call);
+    info->cur_call = info->first_call = -1;
+
     // Set defualt filter text if configured
-    if (get_option_value("cl.filter")) {
-        set_field_buffer(info->fields[FLD_LIST_FILTER], 0, get_option_value("cl.filter"));
+    if (setting_get_value(SETTING_CL_FILTER)) {
+        set_field_buffer(info->fields[FLD_LIST_FILTER], 0, setting_get_value(SETTING_CL_FILTER));
         call_list_form_activate(panel, 0);
     }
 
@@ -129,7 +136,7 @@ call_list_destroy(PANEL *panel)
     hide_panel(panel);
 
     // Free its status data
-    if ((info = (call_list_info_t*) panel_userptr(panel))) {
+    if ((info = call_list_info(panel))) {
 
         // Deallocate forms data
         if (info->form) {
@@ -143,9 +150,12 @@ call_list_destroy(PANEL *panel)
             call_group_destroy(info->group);
         free(info);
     }
+}
 
-    // Finally free the panel memory
-    del_panel(panel);
+call_list_info_t *
+call_list_info(PANEL *panel)
+{
+    return (call_list_info_t*) panel_userptr(panel);
 }
 
 int
@@ -154,7 +164,7 @@ call_list_resize(PANEL *panel)
     int maxx, maxy;
 
     // Get panel info
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
+    call_list_info_t *info = call_list_info(panel);
     // Get current screen dimensions
     getmaxyx(stdscr, maxy, maxx);
 
@@ -175,7 +185,7 @@ call_list_draw_header(PANEL *panel)
     int height, width, colpos, collen, i;
 
     // Get panel info
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
+    call_list_info_t *info = call_list_info(panel);
 
     // Let's draw the fixed elements of the screen
     WINDOW *win = panel_window(panel);
@@ -213,7 +223,7 @@ call_list_draw_header(PANEL *panel)
     wattroff(win, A_BOLD | A_REVERSE | COLOR_PAIR(CP_DEF_ON_CYAN));
 
     // Get filter call counters
-    filter_stats(&info->callcnt, &info->dispcallcnt);
+    sip_calls_stats(&info->callcnt, &info->dispcallcnt);
 
     // Print calls count (also filtered)
     mvwprintw(win, 1, 35, "%*s", 35, "");
@@ -236,8 +246,8 @@ call_list_draw_footer(PANEL *panel)
         key_action_key_str(ACTION_DISP_FILTER), "Search",
         key_action_key_str(ACTION_SHOW_FLOW_EX), "Extended",
         key_action_key_str(ACTION_CLEAR_CALLS), "Clear",
-        key_action_key_str(ACTION_SHOW_RAW), "Raw",
         key_action_key_str(ACTION_SHOW_FILTERS), "Filter",
+        key_action_key_str(ACTION_SHOW_SETTINGS), "Settings",
         key_action_key_str(ACTION_SHOW_COLUMNS), "Columns"
     };
 
@@ -254,24 +264,28 @@ call_list_draw_list(PANEL *panel)
     WINDOW *win;
 
     // Get panel info
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
-
-    // No calls, we've finished drawing the list
-    if (info->dispcallcnt == 0)
-        return;
+    call_list_info_t *info = call_list_info(panel);
 
     // Get window of call list panel
     win = info->list_win;
     getmaxyx(win, height, width);
 
     // If no active call, use the fist one (if exists)
-    if (!info->first_call && call_get_next_filtered(NULL)) {
-        info->cur_call = info->first_call = call_get_next_filtered(NULL);
+    if (info->first_call == -1 && vector_iterator_count(&info->calls)) {
+        vector_iterator_reset(&info->calls);
+        call = vector_iterator_next(&info->calls);
+        info->cur_call = info->first_call = vector_index(vector_iterator_vector(&info->calls), call);
         info->cur_line = info->first_line = 1;
     }
 
+    // Clear call list before redrawing
+    werase(win);
+
+    // Set the iterator position to the first call
+    vector_iterator_set_current(&info->calls, info->first_call - 1 );
+
     // Fill the call list
-    for (call = info->first_call; call; call = call_get_next_filtered(call)) {
+    while ((call = vector_iterator_next(&info->calls))) {
         // Stop if we have reached the bottom of the list
         if (cline == height)
             break;
@@ -285,7 +299,7 @@ call_list_draw_list(PANEL *panel)
             wattron(win, A_BOLD | COLOR_PAIR(CP_DEFAULT));
 
         // Highlight active call
-        if (call == info->cur_call) {
+        if (call == vector_item(vector_iterator_vector(&info->calls), info->cur_call)) {
             // Reverse colors on monochrome terminals
             if (!has_colors())
                 wattron(win, A_REVERSE);
@@ -310,6 +324,7 @@ call_list_draw_list(PANEL *panel)
     // Draw scrollbar to the right
     draw_vscrollbar(win, info->first_line, info->dispcallcnt, 1);
     wnoutrefresh(info->list_win);
+
 }
 
 int
@@ -336,7 +351,7 @@ call_list_draw(PANEL *panel)
 void
 call_list_form_activate(PANEL *panel, int active)
 {
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
+    call_list_info_t *info = call_list_info(panel);
 
     // Store form state
     info->form_active = active;
@@ -373,24 +388,13 @@ call_list_line_text(PANEL *panel, sip_call_t *call, char *text)
     width = getmaxx(win);
 
     // Get panel info
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
+    call_list_info_t *info = call_list_info(panel);
 
     // Print requested columns
     for (i = 0; i < info->columncnt; i++) {
 
-        // Swappable columns
-        switch (info->columns[i].id) {
-            case SIP_ATTR_SRC:
-            case SIP_ATTR_SRC_HOST:
-                colid = (is_option_enabled("sngrep.displayhost")) ? SIP_ATTR_SRC_HOST : SIP_ATTR_SRC;
-                break;
-            case SIP_ATTR_DST:
-            case SIP_ATTR_DST_HOST:
-                colid = (is_option_enabled("sngrep.displayhost")) ? SIP_ATTR_DST_HOST : SIP_ATTR_DST;
-                break;
-            default:
-                colid = info->columns[i].id;
-        }
+        // Get current column id
+        colid = info->columns[i].id;
 
         // Get current column width
         collen = info->columns[i].width;
@@ -419,14 +423,15 @@ call_list_line_text(PANEL *panel, sip_call_t *call, char *text)
 int
 call_list_handle_key(PANEL *panel, int key)
 {
-    int i, height, width, rnpag_steps = get_option_int_value("cl.scrollstep");
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
+    int i, height, width, rnpag_steps = setting_get_intvalue(SETTING_CL_SCROLLSTEP);
+    call_list_info_t *info;
     ui_t *next_panel;
     sip_call_group_t *group;
     int action = -1;
+    sip_call_t *call;
 
     // Sanity check, this should not happen
-    if (!info)
+    if (!(info  = call_list_info(panel)))
         return -1;
 
     // Handle form key
@@ -437,29 +442,34 @@ call_list_handle_key(PANEL *panel, int key)
     WINDOW *win = info->list_win;
     getmaxyx(win, height, width);
 
+    // Reset iterator position to current call
+    vector_iterator_set_current(&info->calls, info->cur_call);
+
     // Check actions for this key
     while ((action = key_find_action(key, action)) != ERR) {
         // Check if we handle this action
         switch (action) {
             case ACTION_DOWN:
                 // Check if there is a call below us
-                if (!info->cur_call || !call_get_next_filtered(info->cur_call))
+                if (!vector_iterator_next(&info->calls))
                     break;
-                info->cur_call = call_get_next_filtered(info->cur_call);
+                info->cur_call = vector_iterator_current(&info->calls);
                 info->cur_line++;
                 // If we are out of the bottom of the displayed list
                 // refresh it starting in the next call
                 if (info->cur_line > height) {
-                    info->first_call = call_get_next_filtered(info->first_call);
+                    vector_iterator_set_current(&info->calls, info->first_call);
+                    vector_iterator_next(&info->calls);
+                    info->first_call = vector_iterator_current(&info->calls);
                     info->first_line++;
                     info->cur_line = height;
                 }
                 break;
             case ACTION_UP:
                 // Check if there is a call above us
-                if (!info->cur_call || !call_get_prev_filtered(info->cur_call))
+                if (!vector_iterator_prev(&info->calls))
                     break;
-                info->cur_call = call_get_prev_filtered(info->cur_call);
+                info->cur_call = vector_iterator_current(&info->calls);
                 info->cur_line--;
                 // If we are out of the top of the displayed list
                 // refresh it starting in the previous (in fact current) call
@@ -490,69 +500,58 @@ call_list_handle_key(PANEL *panel, int key)
                 call_list_form_activate(panel, 1);
                 break;
             case ACTION_SHOW_FLOW:
-                // Display current call flow
-                if (!info->cur_call)
+                // Check we have calls in the list
+                if (info->cur_call == -1)
                     break;
-                next_panel = ui_create(ui_find_by_type(PANEL_CALL_FLOW));
-                if (info->group->callcnt) {
+                ui_create_panel(PANEL_CALL_FLOW);
+                if (call_group_count(info->group)) {
                     group = info->group;
                 } else {
-                    if (!info->cur_call)
-                        break;
                     group = call_group_create();
-                    call_group_add(group, info->cur_call);
+                    call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
+                    call_group_add(group, call);
                 }
                 call_flow_set_group(group);
-                wait_for_input(next_panel);
                 break;
             case ACTION_SHOW_FLOW_EX:
+                // Check we have calls in the list
+                if (info->cur_call == -1)
+                    break;
                 // Display current call flow (extended)
-                next_panel = ui_create(ui_find_by_type(PANEL_CALL_FLOW));
-                if (info->group->callcnt) {
+                ui_create_panel(PANEL_CALL_FLOW);
+                if (call_group_count(info->group)) {
                     group = info->group;
                 } else {
-                    if (!info->cur_call)
-                        break;
                     group = call_group_create();
-                    call_group_add(group, info->cur_call);
-                    call_group_add(group, call_get_xcall(info->cur_call));
+                    call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
+                    call_group_add(group, call);
+                    call_group_add(group, call_get_xcall(call));
                 }
                 call_flow_set_group(group);
-                wait_for_input(next_panel);
                 break;
             case ACTION_SHOW_RAW:
-                // KEY_R , Display current call flow (extended)
-                next_panel = ui_create(ui_find_by_type(PANEL_CALL_RAW));
-                if (info->group->callcnt) {
+                // Check we have calls in the list
+                if (info->cur_call == -1)
+                    break;
+                ui_create_panel(PANEL_CALL_RAW);
+                if (call_group_count(info->group)) {
                     group = info->group;
                 } else {
-                    if (!info->cur_call)
-                        break;
                     group = call_group_create();
-                    call_group_add(group, info->cur_call);
+                    call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
+                    call_group_add(group, call);
                 }
                 call_raw_set_group(group);
-                wait_for_input(next_panel);
                 break;
             case ACTION_SHOW_FILTERS:
-                // KEY_F, Display filter panel
-                next_panel = ui_create(ui_find_by_type(PANEL_FILTER));
-                wait_for_input(next_panel);
-                call_list_clear(panel);
+                ui_create_panel(PANEL_FILTER);
                 break;
             case ACTION_SHOW_COLUMNS:
-                // Display column selection panel
-                next_panel = ui_create(ui_find_by_type(PANEL_COLUMN_SELECT));
-                wait_for_input(next_panel);
-                call_list_clear(panel);
+                ui_create_panel(PANEL_COLUMN_SELECT);
                 break;
             case ACTION_SAVE:
-                if (!is_option_disabled("sngrep.tmpfile")) {
-                    // KEY_S, Display save panel
-                    next_panel = ui_create(ui_find_by_type(PANEL_SAVE));
-                    save_set_group(ui_get_panel(next_panel), info->group);
-                    wait_for_input(next_panel);
-                }
+                next_panel = ui_create_panel(PANEL_SAVE);
+                save_set_group(ui_get_panel(next_panel), info->group);
                 break;
             case ACTION_DISP_INVITE:
                 // Set Display filter text
@@ -567,19 +566,23 @@ call_list_handle_key(PANEL *panel, int key)
                 // Clear List
                 call_list_clear(panel);
                 break;
+            case ACTION_SHOW_SETTINGS:
+                ui_create_panel(PANEL_SETTINGS);
+                break;
             case ACTION_SELECT:
-                if (!info->cur_call)
-                    break;
-                if (call_group_exists(info->group, info->cur_call)) {
-                    call_group_del(info->group, info->cur_call);
+                call = vector_item(vector_iterator_vector(&info->calls), info->cur_call);
+                if (call_group_exists(info->group, call)) {
+                    call_group_del(info->group, call);
                 } else {
-                    call_group_add(info->group, info->cur_call);
+                    call_group_add(info->group, call);
                 }
                 break;
             case ACTION_PREV_SCREEN:
                 // Handle quit from this screen unless requested
-                if (!is_option_enabled("cl.noexitprompt")) {
+                if (setting_enabled(SETTING_EXITPROMPT)) {
                     return call_list_exit_confirm(panel);
+                } else {
+                    return KEY_ESC;
                 }
                 break;
             default:
@@ -603,7 +606,7 @@ call_list_handle_form_key(PANEL *panel, int key)
     int action = -1;
 
     // Get panel information
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
+    call_list_info_t *info = call_list_info(panel);
 
     // Get current field id
     field_idx = field_index(current_field(info->form));
@@ -683,14 +686,12 @@ int
 call_list_help(PANEL *panel)
 {
     WINDOW *help_win;
-    PANEL *help_panel;
     int height, width;
 
     // Create a new panel and show centered
     height = 28;
     width = 65;
     help_win = newwin(height, width, (LINES - height) / 2, (COLS - width) / 2);
-    help_panel = new_panel(help_win);
 
     // Set the window title
     mvwprintw(help_win, 1, 25, "Call List Help");
@@ -739,8 +740,7 @@ call_list_help(PANEL *panel)
 
     // Press any key to close
     wgetch(help_win);
-    update_panels();
-    doupdate();
+    delwin(help_win);
 
     return 0;
 }
@@ -749,14 +749,13 @@ int
 call_list_exit_confirm(PANEL *panel)
 {
     WINDOW *exit_win;
-    PANEL *exit_panel;
     int c;
+
     // Initial exit status
-    int exit = get_option_int_value("cl.defexitbutton");
+    int exit = 1;
 
     // Create a new panel and show centered
     exit_win = newwin(8, 40, (LINES - 8) / 2, (COLS - 40) / 2);
-    exit_panel = new_panel(exit_win);
     keypad(exit_win, TRUE);
 
     // Set the window title
@@ -788,9 +787,6 @@ call_list_exit_confirm(PANEL *panel)
         mvwprintw(exit_win, 6, 20, "[  No   ]");
         wattroff(exit_win, A_REVERSE);
 
-        update_panels();
-        doupdate();
-
         c = wgetch(exit_win);
         switch (c) {
             case KEY_RIGHT:
@@ -799,13 +795,16 @@ call_list_exit_confirm(PANEL *panel)
             case KEY_LEFT:
                 exit = 1;
                 break;
-            case 9:
+            case KEY_TAB:
                 exit = (exit) ? 0 : 1;
                 break;
-            case 10:
+            case KEY_SPACE:
+            case KEY_ENTER:
+            case KEY_INTRO:
+                delwin(exit_win);
                 // If we return ESC, we let ui_manager to handle this
                 // key and exit sngrep gracefully
-                return (exit) ? 27 : 0;
+                return (exit) ? KEY_ESC : 0;
         }
     }
 
@@ -816,8 +815,9 @@ int
 call_list_add_column(PANEL *panel, enum sip_attr_id id, const char* attr, const char *title,
                      int width)
 {
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
-    if (!info)
+    call_list_info_t *info;
+
+    if (!(info = call_list_info(panel)))
         return 1;
 
     info->columns[info->columncnt].id = id;
@@ -831,17 +831,17 @@ call_list_add_column(PANEL *panel, enum sip_attr_id id, const char* attr, const
 void
 call_list_clear(PANEL *panel)
 {
+    call_list_info_t *info;
+
     // Get panel info
-    call_list_info_t *info = (call_list_info_t*) panel_userptr(panel);
-    if (!info)
+    if (!(info = call_list_info(panel)))
         return;
 
     // Initialize structures
-    info->first_call = info->cur_call = NULL;
+    info->first_call = info->cur_call = -1;
     info->first_line = info->cur_line = 0;
-    info->group->callcnt = 0;
+    vector_clear(info->group->calls);
 
     // Clear Displayed lines
     werase(info->list_win);
-    wnoutrefresh(info->list_win);
 }
