@@ -27,21 +27,22 @@
  *
  */
 #include <ctype.h>
+#include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include <stdlib.h>
 #include <locale.h>
-#include "option.h"
+#include "setting.h"
 #include "ui_manager.h"
 #include "capture.h"
 #include "ui_call_list.h"
 #include "ui_call_flow.h"
 #include "ui_call_raw.h"
 #include "ui_filter.h"
-#include "ui_save_pcap.h"
-#include "ui_save_raw.h"
 #include "ui_msg_diff.h"
 #include "ui_column_select.h"
+#include "ui_save.h"
+#include "ui_settings.h"
 
 /**
  * @brief Available panel windows list
@@ -50,15 +51,16 @@
  * and pointer to their main functions.
 
  */
-static ui_t *panel_pool[] = {
+static ui_t *panel_pool[] =
+{
     &ui_call_list,
     &ui_call_flow,
     &ui_call_raw,
     &ui_filter,
     &ui_save,
-    &ui_save_raw,
     &ui_msg_diff,
     &ui_column_select,
+    &ui_settings
 };
 
 int
@@ -76,7 +78,7 @@ init_interface()
     }
 
     // Check if user wants a black background
-    if (is_option_value("background", "dark")) {
+    if (setting_has_value(SETTING_BACKGROUND, "dark")) {
         assume_default_colors(COLOR_WHITE, COLOR_BLACK);
     } else {
         use_default_colors();
@@ -91,13 +93,11 @@ init_interface()
     curs_set(0);
     // Only delay ESC Sequences 25 ms (we dont want Escape sequences)
     ESCDELAY = 25;
-    // Set character input timeout 200 ms
-    halfdelay(REFRESHTHSECS);
 
     // Redefine some keys
     term = getenv("TERM");
     if (term
-        && (!strcmp(term, "xterm") || !strcmp(term, "xterm-color") || !strcmp(term, "vt220"))) {
+            && (!strcmp(term, "xterm") || !strcmp(term, "xterm-color") || !strcmp(term, "vt220"))) {
         define_key("\033[H", KEY_HOME);
         define_key("\033[F", KEY_END);
         define_key("\033OP", KEY_F(1));
@@ -111,7 +111,7 @@ init_interface()
         define_key("\033[17;2~", KEY_F(18));
     }
 
-    if (is_option_value("background", "dark")) {
+    if (setting_has_value(SETTING_BACKGROUND, "dark")) {
         fg = COLOR_WHITE;
         bg = COLOR_BLACK;
     } else {
@@ -163,6 +163,13 @@ ui_create(ui_t *ui)
     return ui;
 }
 
+ui_t *
+ui_create_panel(enum panel_types type)
+{
+    // Find the panel of given type and create it
+    return ui_create(ui_find_by_type(type));
+}
+
 void
 ui_destroy(ui_t *ui)
 {
@@ -175,6 +182,8 @@ ui_destroy(ui_t *ui)
     if (ui->destroy)
         ui->destroy(panel);
 
+    // Deallocate panel pointer
+    del_panel(ui->panel);
     // Initialize panel pointer
     ui->panel = NULL;
 }
@@ -189,24 +198,27 @@ ui_get_panel(ui_t *ui)
 int
 ui_draw_panel(ui_t *ui)
 {
+    PANEL *panel = NULL;
     int ret = 0;
 
     //! Sanity check, this should not happen
     if (!ui)
         return -1;
 
-    // Make this panel the topmost panel
-    top_panel(ui_get_panel(ui));
+    // Get ui panel pointer
+    if (!(panel = ui_get_panel(ui)))
+        return -1;
+
+    // Set character input timeout 200 ms
+    halfdelay(REFRESHTHSECS);
 
     // Request the panel to draw on the scren
     if (ui->draw) {
-        ret = ui->draw(ui_get_panel(ui));
+        ret = ui->draw(panel);
+    } else {
+        touchwin(panel_window(panel));
     }
 
-    // Update panel stack
-    update_panels();
-    doupdate();
-
     return ret;
 }
 
@@ -233,12 +245,11 @@ ui_help(ui_t *ui)
     // Disable input timeout
     nocbreak();
     cbreak();
+
     // If current ui has help function
     if (ui->help) {
         ui->help(ui_get_panel(ui));
     }
-    // Enable input timeout
-    halfdelay(REFRESHTHSECS);
 }
 
 int
@@ -265,7 +276,7 @@ ui_find_by_panel(PANEL *panel)
 }
 
 ui_t *
-ui_find_by_type(int type)
+ui_find_by_type(enum panel_types type)
 {
     int i;
     // Return ui pointer if found
@@ -277,18 +288,34 @@ ui_find_by_type(int type)
 }
 
 int
-wait_for_input(ui_t *ui)
+wait_for_input()
 {
+    ui_t *ui;
     WINDOW *win;
+    PANEL *panel;
 
-    // Keep getting keys until panel is destroyed
-    while (ui_get_panel(ui)) {
-        // Redraw this panel
-        if (ui_draw_panel(ui) != 0)
-            return -1;
+    // While there are still panels
+    while (panel_below(NULL)) {
+
+        // Redraw all panels
+        panel = NULL;
+        while ((panel = panel_above(panel))) {
+            // Get panel interface structure
+            ui = ui_find_by_panel(panel);
+            // Redraw this panel
+            if (ui_draw_panel(ui) != 0)
+                return -1;
+        }
+
+        // Update panel stack
+        update_panels();
+        doupdate();
+
+        // Get topmost panel
+        panel = panel_below(NULL);
 
         // Enable key input on current panel
-        win = panel_window(ui_get_panel(ui));
+        win = panel_window(panel);
         keypad(win, TRUE);
 
         // Get pressed key
@@ -324,25 +351,22 @@ default_handle_key(ui_t *ui, int key)
                 ui_resize_panel(ui);
                 break;
             case ACTION_TOGGLE_SYNTAX:
-                toggle_option("syntax");
+                setting_toggle(SETTING_SYNTAX);
                 break;
             case ACTION_TOGGLE_HINT:
-                toggle_option("hintkeyalt");
+                setting_toggle(SETTING_ALTKEY_HINT);
                 break;
             case ACTION_CYCLE_COLOR:
-                if (is_option_enabled("color.request")) {
-                    toggle_option("color.request");
-                    toggle_option("color.callid");
-                } else if (is_option_enabled("color.callid")) {
-                    toggle_option("color.callid");
-                    toggle_option("color.cseq");
-                } else if (is_option_enabled("color.cseq")) {
-                    toggle_option("color.cseq");
-                    toggle_option("color.request");
-                }
+                setting_toggle(SETTING_COLORMODE);
                 break;
             case ACTION_SHOW_HOSTNAMES:
-                toggle_option("sngrep.displayhost");
+                setting_toggle(SETTING_DISPLAY_HOST);
+                break;
+            case ACTION_SHOW_ALIAS:
+                setting_toggle(SETTING_DISPLAY_ALIAS);
+                break;
+            case ACTION_SHOW_SETTINGS:
+                ui_create_panel(PANEL_SETTINGS);
                 break;
             case ACTION_TOGGLE_PAUSE:
                 // Pause/Resume capture
@@ -398,7 +422,7 @@ draw_keybindings(PANEL *panel, const char *keybindings[], int count)
 
     // Write a line all the footer width
     wattron(win, COLOR_PAIR(CP_DEF_ON_CYAN));
-    clear_line(win, height-1);
+    clear_line(win, height - 1);
 
     // Draw keys and their actions
     for (key = 0; key < count; key += 2) {
@@ -489,7 +513,7 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
 {
     int height, width, line, column, i;
     char *cur_line = msg->payload;
-    int syntax = is_option_enabled("syntax");
+    int syntax = setting_enabled(SETTING_SYNTAX);
 
     // Default text format
     int attrs = A_NORMAL | COLOR_PAIR(CP_DEFAULT);
@@ -536,13 +560,13 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
                 // tag and branch syntax
                 if (i > 0 && msg->payload[i - 1] == ';') {
                     // Highlight branch if requested
-                    if (is_option_enabled("syntax.branch")) {
+                    if (setting_enabled(SETTING_SYNTAX_BRANCH)) {
                         if (!strncasecmp(msg->payload + i, "branch", 6)) {
                             attrs = A_BOLD | COLOR_PAIR(CP_CYAN_ON_DEF);
                         }
                     }
                     // Highlight tag if requested
-                    if (is_option_enabled("syntax.tag")) {
+                    if (setting_enabled(SETTING_SYNTAX_TAG)) {
                         if (!strncasecmp(msg->payload + i, "tag", 3)) {
                             if (!strncasecmp(cur_line, "From:", 5)) {
                                 attrs = A_BOLD | COLOR_PAIR(CP_DEFAULT);
@@ -600,3 +624,60 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
 
     return line - starting;
 }
+
+int
+dialog_run(const char *fmt, ...)
+{
+    char textva[2048];
+    va_list ap;
+    int height, width;
+    WINDOW *win;
+    char *word;
+    int col = 2;
+    int line = 2;
+
+    // Get the message from the format string
+    va_start(ap, fmt);
+    vsprintf(textva, fmt, ap);
+    va_end(ap);
+
+    // Determine dialog dimensions
+    height = 6 + (strlen(textva) / 50);
+    width = strlen(textva);
+
+    // Check we don't have a too big or small window
+    if (width > DIALOG_MAX_WIDTH)
+        width = DIALOG_MAX_WIDTH;
+    if (width < DIALOG_MIN_WIDTH)
+        width = DIALOG_MIN_WIDTH;
+
+    // Create the window
+    win = newwin(height, width, (LINES - height) / 2, (COLS - width) / 2);
+    box(win, 0, 0);
+
+    // Write the message into the screen
+    for (word = strtok(textva, " "); word; word = strtok(NULL, " ")) {
+        if (col + strlen(word) > width - 2) {
+            line++;
+            col = 2;
+        }
+        mvwprintw(win, line, col, "%s", word);
+        col += strlen(word) + 1;
+    }
+
+    // Write Accept button
+    wattron(win, A_REVERSE);
+    mvwprintw(win, height - 2, width/2 - 5, "[ Accept ]");
+
+    curs_set(0);
+    // Disable input timeout
+    nocbreak();
+    cbreak();
+
+    // Wait for input
+    wgetch(win);
+
+    delwin(win);
+    return 1;
+
+}
